---@meta
--[[
    This file is not loaded, it is here to provide code completion in VSCode
]]

_G.Global = {}
_G.tweak_data = {}
---@class EHITweakData
_G.tweak_data.ehi = {}
_G.managers = {}
---@type boolean
_G.IS_VR = ...
---@class TimerGui
_G.TimerGui = {}
---@class DigitalGui
_G.DigitalGui = {}
---@class ExperienceManager
_G.ExperienceManager = {}
---@param o table? Can be used to provide `self` to the callback function
---@param base_callback_class table
---@param base_callback_func_name string
---@param base_callback_param any?
---@return function
_G.callback = function(o, base_callback_class, base_callback_func_name, base_callback_param)
end
---@return Vector3
---@overload fun(x: number, y: number, z: number): Vector3
_G.Vector3 = function()
end
---@return Rotation
---@overload fun(x: number, y: number): Rotation
---@overload fun(x: number, y: number, z: number): Rotation
---@overload fun(x: number, y: number, z: number, w: number): Rotation
_G.Rotation = function()
end
---@generic T
---@param TC T
---@return T
_G.deep_clone = function(TC)
end
---@generic T
---@param TC T
---@return T
CoreTable.deep_clone = function(TC)
end

---@generic T: table
---@param super T? A base class which `class` will derive from
---@return T
function class(super) end

---@class managers Global table of all managers in the game
---@field experience ExperienceManager
---@field game_play_central GamePlayCentralManager
---@field hud HUDManager
---@field mission MissionManager
---@field localization LocalizationManager
---@field loot LootManager
---@field worlddefinition WorldDefinition
---@field [unknown] unknown

---@class tweak_data Global table of all configuration data
---@field levels LevelsTweakData
---@field ehi EHITweakData
---@field [unknown] unknown

---@class Global_game_settings
---@field difficulty string
---@field gamemode string
---@field level_id string
---@field single_player boolean
---@field team_ai boolean

---@class Global
---@field achievment_manager table
---@field block_update_outfit_information boolean
---@field editor_mode boolean Only in `Beardlib Editor`
---@field load_level boolean
---@field hud_disabled boolean
---@field game_settings Global_game_settings
---@field mission_manager table
---@field statistics_manager table
---@field wallet_panel Panel?

---@class _G Global
---@field Global Global
---@field managers managers Global table of all managers in the game
---@field tweak_data tweak_data Global table of all configuration data
---@field PrintTableDeep fun(tbl: table, maxDepth: integer?, allowLogHeavyTables: boolean?, customNameForInitialLog: string?, tablesToIgnore: table|string?, skipFunctions: boolean?) Recursively prints tables; depends on mod: https://modworkshop.net/mod/34161
---@field PrintTable fun(tbl: table) Prints tables, provided by SuperBLT

---@class mathlib
---@field lerp fun(a: number, b: number, lerp: number): number Linearly interpolates between `a` and `b` by `lerp`
---@field round fun(n: number, precision: number?): number Rounds number with precision
---@field clamp fun(number: number, min: number, max: number): number Returns `number` clamped to the inclusive range of `min` and `max`
---@field rand fun(a: number, b: number?): number If `b` is provided, returns random number between `a` and `b`. Otherwise returns number between `0` and `a`
---@field mod fun(n: number, div: number): number Returns remainder of a division

---@class tablelib
---@field size fun(tbl: table): number Returns size of the table
---@field contains fun(v: table, e: string): boolean Returns `true` or `false` if `e` exists in the table
---@field index_of fun(v: table, e: string): integer Returns `index` of the element when found, otherwise `-1` is returned
---@field get_key fun(map: table, wanted_key_value: any): any? Returns `key name` if value exists
---@field list_to_set fun(list: table): table Maps values as keys

---@class InteractionExt
---@field interact_position fun(): Vector3

---@class Unit
---@field base unknown
---@field timer_gui fun(): TimerGui
---@field digital_gui fun(): DigitalGui
---@field interaction fun(): InteractionExt
---@field mission_door_device fun(): MissionDoorDevice
---@field key fun(): string
---@field editor_id fun(): number
---@field position fun(): Vector3
---@field [unknown] unknown

---@class LocalizationManager
---@field exists fun(self: self, string_id: string): boolean SuperBLT only
---@field text fun(self: self, string_id: string, macros: table?): string
---@field to_upper_text fun(self: self, string_id: string, macros: table?): string

---@class PanelBaseObject
---@field x fun(self: self): number
---@field set_x fun(self: self, x: number)
---@field y fun(self: self): number
---@field set_y fun(self: self, y: number)
---@field w fun(self: self): number
---@field set_w fun(self: self, w: number)
---@field h fun(self: self): number
---@field set_h fun(self: self, h: number)
---@field top fun(self: self): number
---@field set_top fun(self: self, top: number): number
---@field bottom fun(self: self): number
---@field set_bottom fun(self: self, bottom: number): number
---@field left fun(self: self): number
---@field set_left fun(self: self, left: number): number
---@field right fun(self: self): number
---@field set_right fun(self: self, right: number): number
---@field center_x fun(self: self): number
---@field set_center_x fun(self: self, center_x: number)
---@field set_position fun(self: self, x: number, y: number)
---@field set_leftbottom fun(self: self, left: number, bottom: number)
---@field stop fun(self: self, anim_thread: thread?)
---@field animate fun(self: self, f: function, ...:any?): thread

---@class Panel : PanelBaseObject
---@field child fun(self: self, child_name: string): (PanelText|PanelBitmap|PanelRectangle|self)?
---@field remove fun(self: self, child_name: PanelBaseObject)
---@field text fun(self: self, params: table): PanelText
---@field bitmap fun(self: self, params: table): PanelBitmap
---@field rect fun(self: self, params: table): PanelRectangle
---@field panel fun(self: self, params: table): self
---@field children fun(self: self): table Returns an ipairs table of all items created on the panel

---@class PanelText : PanelBaseObject
---@field set_color fun(self: self, color: number)
---@field set_text fun(self: self, text: string)
---@field text_rect fun(self: self): x: number, y: number, w: number, h: number Returns rectangle of the text
---@field font_size fun(self: self): number
---@field set_font_size fun(self: self, font_size: number)
---@field visible fun(self: self): boolean
---@field set_visible fun(self: self, visibility: boolean)
---@field set_size fun(self: self, w: number, h: number)

---@class PanelBitmap : PanelBaseObject
---@field set_color fun(self: self, color: number)
---@field set_image fun(self: self, texture_path: string, texture_rect_x: number?, texture_rect_y: number?, texture_rect_w: number?, texture_rect_h: number?)
---@field set_visible fun(self: self, visibility: boolean)
---@field set_texture_rect fun(self: self, x: number, y: number, w: number, h: number)
---@field set_size fun(self: self, w: number, h: number)

---@class PanelRectangle : PanelBaseObject
---@field set_color fun(self: self, color: number)
---@field set_size fun(self: self, w: number, h: number)