local CS = CSHUD
CSRadioManager = class()
function CSRadioManager:init()
    if blt.xaudio and CS:GetOption("play_sounds") then
        blt.xaudio.setup()
        local win_faction = CS._cache.player_faction
        local loss_faction = CS._cache.enemy_faction
        self._player_faction = win_faction
        self._sounds =
        {
            win = XAudio.Buffer:new(CS.AssetsPath .. "sounds/pd2_hud_cs/" .. win_faction .. "_win.ogg"),
            loss = XAudio.Buffer:new(CS.AssetsPath .. "sounds/pd2_hud_cs/" .. loss_faction .. "_win.ogg"),
            bomb_planted = XAudio.Buffer:new(CS.AssetsPath .. "sounds/pd2_hud_cs/bomb_planted.ogg"),
            bomb_defused = XAudio.Buffer:new(CS.AssetsPath .. "sounds/pd2_hud_cs/bomb_defused.ogg"),
        }
        self._sound_source = XAudio.Source:new()
        if self._sound_source.set_volume then
            self._sound_source:set_volume(1)
        end
        self:SetSingleSound(false)
    else
        self._sounds = {}
        self._player_faction = CS._cache.player_faction
    end
end

function CSRadioManager:init_finalize()
    if CS:GetModOption("bai", "bomb_on_endless") and BAI then
        local level_id = CS._cache.level_id
        if not CS:IsOr(level_id, "hox_1", "pines") then
            local hud = managers.hud
            if level_id == "pbr" then -- Beneath the Mountain
                local function f()
                    if hud._hud_objectives._active_objective_id == "heist_jerry01_20" then
                        self:PlayBombPlanted()
                        hud:ShowBombMessage()
                    end
                end
                BAI:AddEvent(BAI.EventList.EndlessAssaultStart, f)
            else
                local function f()
                    self:PlayBombPlanted()
                    hud:ShowBombMessage()
                end
                BAI:AddEvent(BAI.EventList.EndlessAssaultStart, f)
            end
        end
    end
    if self._sound_source then
        local sfx_volume = managers.user:get_setting("sfx_volume") / 100
        local music_volume = managers.user:get_setting("music_volume") / 100
        if music_volume > sfx_volume then
            self._sound_source:set_type(XAudio.Source.MUSIC) -- By default, all XAudio sources are set as Sound
        end
    end
end

function CSRadioManager:PlaySound(sound)
    local buffer = self._sounds[sound]
    if not buffer then
        return
    end
    if not self._sound_source:is_closed() then
        self._sound_source:stop()
        self._sound_source:set_buffer(buffer)
        self._sound_source:play()
    end
end

function CSRadioManager:PlaySoundLast(sound)
    self:SetSingleSound(true)
    self:PlaySound(sound)
end

function CSRadioManager:SetSingleSound(single)
    if self._sound_source and not self._sound_source:is_closed() then
        self._sound_source:set_single_sound(single)
    end
end

function CSRadioManager:PlayWin()
    if self._bomb_planted then
        if self._player_faction == "t" then
            self:PlaySoundLast("win")
        else -- CT
            self:PlaySound("bomb_defused")
            local function DelayWinSound()
                self:PlaySoundLast("win")
            end
            CS:DelayCall("DelayWinSound", 2, DelayWinSound)
        end
    else
        self:PlaySoundLast("win")
    end
end

function CSRadioManager:PlayLoss()
    if self._bomb_planted then
        if self._player_faction == "t" then
            self:PlaySound("bomb_defused")
            local function DelayLossSound()
                self:PlaySoundLast("loss")
            end
            CS:DelayCall("DelayLossSound", 2, DelayLossSound)
        else -- CT
            self:PlaySoundLast("loss")
        end
    else
        self:PlaySoundLast("loss")
    end
end

function CSRadioManager:PlayBombPlanted()
    self._bomb_planted = true
    self:PlaySound("bomb_planted")
end

function CSRadioManager:PlayBombDefused()
    self._bomb_planted = false
    self:PlaySound("bomb_defused")
end