local CSHUD = CSHUD
local mvp = "MVP: "
local function GetPeerName(peer_id)
    local peer = managers.network:session():peer(peer_id)
    local name = peer and peer:name() or "Player"
    return name
end

local function FormatNumbers(n, one, two_or_more)
    if n == 1 then
        return one or "once"
    else
        return two_or_more or (n .. " times")
    end
end

function CSHUD:ShowFunFact()
    if self.IsClient or self._cache.FunFactSent or self._cache.SinglePlayer then
        return
    end
    if self:GetOption("announce_winning_faction") then
        self:SendMessage(self._cache.winning_faction)
    end
    self:GetKillMVP()
    self:AddKillsFunFact()
    self:AddTimeFunFact()
    self:AddOtherFunFact()
    if self:GetOption("announce_mvp") and self._cache.success then
        self:SendMessage(mvp)
    end
    local r = self:ChooseRandomFunFact()
    if r and self:GetOption("announce_funfact") then
        self:SendMessage(r)
    end
    self._cache.FunFactSent = true
end

function CSHUD:ChooseRandomFunFact()
    if #self.FunFacts == 0 then
        return nil
    end
    return self.FunFacts[math.random(#self.FunFacts)]
end

function CSHUD:GetKillMVP()
    if not self._cache.success then
        return
    end
    if math.random(100) % 2 == 0 then -- Normal kills
        mvp = mvp .. string.format(managers.localization:text("csgo_mvp_most_eliminations"), GetPeerName(self._cache.FunFacts.MostKillsPeerID))
    else -- Special kills
        mvp = mvp .. string.format(managers.localization:text("csgo_mvp_most_special_eliminations"), GetPeerName(self._cache.FunFacts.MostSpecialKillsPeerID))
    end
    
end

function CSHUD:AddKillsFunFact()
    local network = managers.network:session()
    -- Common
    local total_kills = managers.statistics:session_total_kills_by_anyone()
    if total_kills ~= 0 then
        self:AddFunFact("Total " .. total_kills .. " units were killed")
    elseif self._cache.success then
        local faction = self._cache.player_faction == "ct" and "Counter-Terrorists" or "Terrorists"
        self:AddFunFact(faction .. " won without any kills")
    end

    -- Team Stats (MP only and with atleast 2 or more players)
    if self._cache.EndGameStats and #self._cache.EndGameStats ~= 1 then
        local EndGameStats = self._cache.EndGameStats
        local IDs = self._cache.EndGameStats_IDs
        -- Select random stat and compare it with others
        local stats =
        {
            "total_kills",
            "total_specials_kills",
            "total_head_shots",
            "accuracy",
            "downs"
        }
        local random_stat = math.random(#stats)
        local random_stat_string = stats[random_stat]
        local message
        if random_stat == 1 then -- Total Kills
            message = "%s had %d total kills while %s had %d total kills"
        elseif random_stat == 2 then -- Total Specials Kills
            message = "%s killed %d special enemies while %s killed %d special enemies"
        elseif random_stat == 3 then -- Total Headshots
            message = "%s scored %d total headshots while %s scored %d total headshots"
        elseif random_stat == 4 then -- Accuracy
            message = "%s had an accuracy of %d%% while %s had an accuracy of %d%%"
        else -- Downs
            message = "%s went down %d time(s) while %s went down %d time(s)"
        end
        local first_player = table.remove(IDs, math.random(#IDs))
        local second_player = IDs[math.random(#IDs)]
        message = string.format(message,
            EndGameStats[first_player].name,
            EndGameStats[first_player][random_stat_string],
            EndGameStats[second_player].name,
            EndGameStats[second_player][random_stat_string])
        self:AddFunFact(message)

        -- Other stats for other players (except player showed in the "Crew Statistics" tab)
        for peer_id, stat in pairs(EndGameStats) do
            if peer_id ~= self._cache.MostKillsPeerID then
                self:AddFunFact(GetPeerName(peer_id) .. " killed " .. stat.total_kills .. " " .. FormatNumbers(stat.total_kills, "enemy", "enemies"))
            end
            if peer_id ~= self._cache.MostSpecialKillsPeerID then
                self:AddFunFact(GetPeerName(peer_id) .. " killed " .. stat.total_specials_kills .. " special " .. FormatNumbers(stat.total_specials_kills, "enemy", "enemies"))
            end
            if peer_id ~= self._cache.BestAccuracyPeerID then
                self:AddFunFact(GetPeerName(peer_id) .. " had an accuracy of " .. stat.accuracy .. "%")
            end
            if peer_id ~= self._cache.MostDownedPeerID then
                self:AddFunFact(GetPeerName(peer_id) .. " got downed " .. stat.downs .. " " .. FormatNumbers(stat.downs, "time", "times"))
            end
        end

        -- Your accuracy vs the rest of the team
        local everyone_except_you = #EndGameStats - 1
        for peer_id, stat_1 in pairs(EndGameStats) do -- First player
            local accuracy = 0
            for peer_id_2, stat_2 in pairs(EndGameStats) do -- Other players
                if peer_id ~= peer_id_2 then
                    accuracy = accuracy + stat_2.accuracy
                end
            end
            accuracy = math.floor(accuracy / everyone_except_you)
            self:AddFunFact(GetPeerName(peer_id) .. " had an accuracy of " .. stat_1.accuracy .. "%, while the rest of the team had an accuracy of " .. accuracy .. "%")
            self:AddFunFact(GetPeerName(peer_id) .. " had an accuracy of " .. stat_1.accuracy .. "%, while their team's was " .. self._cache.FunFacts.GroupAccuracy .. "%")
        end
    end

    -- Special enemies
    local enemies =
    {
        "sniper(s)",
        "bulldozer(s)",
        "shield(s)",
        "medic(s)",
        "cloaker(s)",
        "tazer(s)"
    }

    -- Civilian kills
    for peer_id, value in pairs(self.FunFacts.CivilianKills) do
        if peer_id > 0 and value and value > 0 then
            self:AddFunFact(GetPeerName(peer_id) .. " killed " .. value .. " civilian" .. FormatNumbers(value, "", "s"))
        end
    end
    if self.FunFacts.CivilianKills[-1] then
        local value = self.FunFacts.CivilianKills[-1]
        self:AddFunFact("Environment killed " .. value .. " civilian" .. FormatNumbers(value, "", "s"))
    end
end

function CSHUD:AddTimeFunFact()
    local time = managers.game_play_central and managers.game_play_central:get_heist_timer()
    if time and time <= 600 and self._cache.success then -- 10 minutes or less
        local s = "heist"
        self:AddFunFact("That " .. s .. " took only " .. self:FormatTime(time) .. "!")
    end
end

function CSHUD:AddOtherFunFact()
    -- Last Player Alive
    if self._cache.success then
        if #self._cache.teammates > 1 and #self._cache.in_custody == #self._cache.teammates - 1 then -- One teammate successfully escaped
            for character, data in pairs(self._cache.teammates) do
                if not self._cache.in_custody[character] then -- This character escaped
                    local peer_name = GetPeerName(data.peer_id)
                    mvp = string.format(managers.localization:text("csgo_mvp_last_member"), peer_name)
                    self:AddFunFact(string.format(managers.localization:text("csgo_funfact_last_member"), peer_name))
                    break
                end
            end
        end
    end
    for peer_id, value in pairs(self.FunFacts.InCustodyTimes) do
        if value and value > 0 then
            self:AddFunFact(GetPeerName(peer_id) .. " went to custody " .. FormatNumbers(value))
        end
    end
    for helping_peer_id, tbl in pairs(self.FunFacts.Revive) do
        for hint, _ in pairs(tbl) do
            for helped_peer_id, value in pairs(tbl[hint]) do
                if value and value > 0 then
                    local s_hint = hint == "helpedup" and "revived" or "rescued"
                    self:AddFunFact(GetPeerName(helping_peer_id) .. " " .. s_hint .. " " .. GetPeerName(helped_peer_id) .. " " .. FormatNumbers(value))
                end
            end
        end
    end
    for peer_id, value in pairs(self.FunFacts.GagePackages) do
        if value and value > 0 then
            self:AddFunFact(GetPeerName(peer_id) .. " picked up " .. FormatNumbers(value, "Gage package", "Gage packages"))
        end
    end
end

function CSHUD:AddFunFact(f)
    self.FunFacts[#self.FunFacts + 1] = f
end