local CSHUD = CSHUD
dofile(CSHUD.LuaPath .. "Animations.lua")

local original =
{
    _setup_player_info_hud_pd2 = HUDManager._setup_player_info_hud_pd2,
    show_point_of_no_return_timer = HUDManager.show_point_of_no_return_timer,
    set_teammate_grenades = HUDManager.set_teammate_grenades,
    animate_grenade_flash = HUDManager.animate_grenade_flash,
    set_teammate_grenades_amount = HUDManager.set_teammate_grenades_amount,
    set_cable_ties_amount = HUDManager.set_cable_ties_amount,
    set_teammate_name = HUDManager.set_teammate_name,
    set_teammate_callsign = HUDManager.set_teammate_callsign,
    temp_show_carry_bag = HUDManager.temp_show_carry_bag,
    temp_hide_carry_bag = HUDManager.temp_hide_carry_bag,
    sync_set_assault_mode = HUDManager.sync_set_assault_mode,
    add_teammate_panel = HUDManager.add_teammate_panel,
    sync_start_assault = HUDManager.sync_start_assault,
    sync_end_assault = HUDManager.sync_end_assault,
    _create_heist_timer = HUDManager._create_heist_timer
}

HUDManager.BigLobby = HUDManager.PLAYER_PANEL > 4
local level_id = Global.game_settings.level_id

function HUDManager:_setup_player_info_hud_pd2()
    self._custody_panels_visible = false

    self:InitCustomCode()
    self:InitBigLobby()
    self:InitVR()

    original._setup_player_info_hud_pd2(self)
end

function HUDManager:InitCustomCode()
    self:CreateCustodyTeammatePanels()
    self:CreateStatsScreen()
    self:CreateMessagePanel()
    self:CreateKillfeed()
end

function HUDManager:InitBigLobby()
    if not self.BigLobby then
        return
    end
    -- Disable these function when running BigLobby mod
    local function empty()
    end
    local f =
    {
        "temp_show_carry_bag",
        "temp_hide_carry_bag",
        "set_teammate_carry_info",
        "remove_teammate_carry_info",
        "ai_carry",
        "set_ai_stopped",
        "CustodyTeammatePanelsVisible"
    }
    for _, v in ipairs(f) do
        self[v] = empty
    end
end

function HUDManager:InitVR()
    if not _G.IS_VR then
        return
    end
    -- Disable these function when playing in VR
    local function empty()
    end
    local f =
    {
        "ai_carry",
        "CustodyTeammatePanelsVisible",
        "ChangeManiacVisibility"
    }
    for _, v in ipairs(f) do
        self[v] = empty
    end
end

if false then -- Not currently used
    function HUDManager:UpdateAvatar(i, texture, texture_rect)
        CSHUD:CallListener(CSHUD.ListenerEvent.UpdateAvatar, i, texture, texture_rect)
    end
end

function HUDManager:UpdateAvatar(i, texture, texture_rect)
    self._custody_teammate_panels[i]:UpdateAvatarTexture(texture, texture_rect)
    self._stats_screen_new:UpdateAvatar(i, texture, texture_rect)
end

function HUDManager:set_player_armor(data)
    if data.current == 0 and not data.no_hint then
        managers.hint:show_hint("damage_pad")
    end

    self._teammate_panels[HUDManager.PLAYER_PANEL]:set_armor(data)
    self._stats_screen_new:SetAP(HUDManager.PLAYER_PANEL, data)
end

function HUDManager:show_point_of_no_return_timer(tweak_id)
    original.show_point_of_no_return_timer(self, tweak_id)
    managers.cs_radio:PlayBombPlanted()
    self._hud_heist_timer:SetBombIconVisible(true)
end

if level_id ~= "deep" then
    original.hide_point_of_no_return_timer = HUDManager.hide_point_of_no_return_timer
    function HUDManager:hide_point_of_no_return_timer(...)
        original.hide_point_of_no_return_timer(self)
        managers.cs_radio:PlayBombDefused()
        self._hud_heist_timer:SetBombIconVisible(false)
    end
end

function HUDManager:remove_special_equipment(equipment)
    self._teammate_panels[HUDManager.PLAYER_PANEL]:remove_special_equipment(equipment)
    self._stats_screen_new:RemoveSpecialEquipment(HUDManager.PLAYER_PANEL, equipment)
end

function HUDManager:set_special_equipment_amount(equipment_id, amount)
    self._teammate_panels[HUDManager.PLAYER_PANEL]:set_special_equipment_amount(equipment_id, amount)
end

function HUDManager:set_teammate_name(i, teammate_name)
    original.set_teammate_name(self, i, teammate_name)
    self._custody_teammate_panels[i]:SetName(teammate_name)
    self._stats_screen_new:SetName(i, teammate_name)
end

function HUDManager:set_teammate_callsign(i, id)
    original.set_teammate_callsign(self, i, id)
	self._custody_teammate_panels[i]:SetCallsign(id)
end

function HUDManager:set_teammate_condition(i, icon_data, text)
    self._teammate_panels[i]:set_condition(icon_data, text)
    self._custody_teammate_panels[i]:SetCondition(icon_data)
end

function HUDManager:start_teammate_timer(i, time)
    if self._teammate_panels[i] then
        self._teammate_panels[i]:start_timer(time)
        self._custody_teammate_panels[i]:StartTimer(time)
    end
end

function HUDManager:pause_teammate_timer(i, pause)
    if self._teammate_panels[i] then
        self._teammate_panels[i]:set_pause_timer(pause)
        self._custody_teammate_panels[i]:SetPauseTimer(pause)
    end
end

function HUDManager:stop_teammate_timer(i)
    if self._teammate_panels[i] then
        self._teammate_panels[i]:stop_timer()
        self._custody_teammate_panels[i]:StopTimer()
    end
end

function HUDManager:ai_carry(ai_panel, show)
    if show then
        self._teammate_panels[ai_panel]:set_carry_info()
    else
        self._teammate_panels[ai_panel]:remove_carry_info()
    end
end

function HUDManager:sync_set_assault_mode(mode)
    if CSHUD._cache.assault_mode ~= mode then
        local FF = CSHUD._cache.FunFacts
        CSHUD._cache.assault_mode = mode
        if mode == "phalanx" then
            FF.CaptainArrive = TimerManager:game():time()
        else
            local final_time = TimerManager:game():time() - (FF.CaptainArrive or 0)
            if final_time == 0 then
                CSHUD:AddFunFact("Captain was defeated before he could assemble")
            else
                CSHUD:AddFunFact("Captain survived " .. CSHUD:FormatTime(final_time) .. " before his defeat")
            end
        end
    end
    original.sync_set_assault_mode(self, mode)
end

local function SetCustodyPanelsVisible(self, visible)
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._custody_teammate_panels[i]:SetVisible(visible)
    end
end

function HUDManager:show_stats_screen()
	self._stats_screen_new:Show()

    self._showing_stats_screen = true

    if self._custody_panels_visible then
        SetCustodyPanelsVisible(self, false)
    end

	--self:add_updator("_hud_statsscreen", callback(self._hud_statsscreen, self._hud_statsscreen, "update"))
end

function HUDManager:update_stat_screen()
end

function HUDManager:hide_stats_screen()
	self._showing_stats_screen = false

	if self._stats_screen_new then
		self._stats_screen_new:Hide()
    end

    if self._custody_panels_visible then
        SetCustodyPanelsVisible(self, true)
    end

	--self:remove_updator("_hud_statsscreen")
end

function HUDManager:SetCustody(panel, in_custody)
    panel = panel or 0
    if panel ~= 0 then
        if in_custody then
            local peer_id = nil
            if panel == HUDManager.PLAYER_PANEL then
                peer_id = managers.network:session():local_peer():id()
            elseif not self._teammate_panels[panel]._ai then
                peer_id = self._teammate_panels[panel]._peer_id
            end
            if peer_id then
                CSHUD.FunFacts.InCustodyTimes[peer_id] = (CSHUD.FunFacts.InCustodyTimes[peer_id] or 0) + 1
            end
        end
        if not _G.IS_VR then
            self._teammate_panels[panel]:SetCustody(in_custody)
            self._custody_teammate_panels[panel]:SetCustody(in_custody)
            self._stats_screen_new:SetCustody(panel, in_custody)
        end
        self[in_custody and "UnregisterPlayer" or "RegisterPlayer"](self)
    end
end

function HUDManager:CivilianKilled()
    managers.chat:_receive_message(1, "[+" .. CSHUD:FormatTime(tweak_data.player.damage.respawn_time_penalty, CSHUD.Enum.TimeFormat.Short) .. " Trade Delay]", managers.localization:text("hud_penalty_killed_civilian"), Color.red)
end

function HUDManager:CreateCustodyTeammatePanels()
    self._custody_teammate_panels = {}
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._custody_teammate_panels[i] = HUDCustodyTeammatePanels:new(i, i == HUDManager.PLAYER_PANEL)
    end
end

function HUDManager:CustodyTeammatePanelsVisible(custody)
    self._custody_panels_visible = custody
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._custody_teammate_panels[i]:SetVisible(custody)
    end
end

function HUDManager:CustodyTeammatePanelsWatchingTeammate(u_key)
    local i_watch = 0
    local criminals = managers.criminals._characters or {}
    for _, char in pairs(criminals) do
        if char.name and char.taken and char.unit:key() == u_key then
            i_watch = char.data.panel_id
            break
        end
    end
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._custody_teammate_panels[i]:SetWatching(i_watch)
    end
end

function HUDManager:SetAICallback(i, ai)
end

function HUDManager:CustodyTeammatePanels_SetAI(i, ai)
    self._custody_teammate_panels[i]:SetAI(ai)
end

function HUDManager:CustodyTeammatePanels_SetWeapons(i, peer_id, ai, ai_loadout_params)
    self._custody_teammate_panels[i]:SetWeapons(peer_id, ai, ai_loadout_params)
end

function HUDManager:CustodyTeammatePanels_SetPanelVisible(i, visible)
    self._custody_teammate_panels[i]:SetPanelVisible(visible)
end

function HUDManager:CreateStatsScreen()
    self._stats_screen_new = HUDStatsScreenNew:new()
end

function HUDManager:SetVisibleStatsScreenPanel(i, visible)
    self._stats_screen_new:SetPanelVisible(i, visible)
end

function HUDManager:SetStatsScreenLevel(i, rank, level)
    self._stats_screen_new:SetRank(i, rank, level)
end

function HUDManager:SetStatsScreenPing(i, ping, packet_loss)
    self._stats_screen_new:SetPing(i, ping, packet_loss)
end

function HUDManager:CreateKillfeed()
    if not CSHUD:IsKillfeedEnabled() then
        return
    end
    local hud = self:script(PlayerBase.PLAYER_INFO_HUD_PD2)
    self._killfeed = HUDKillFeed:new(hud)
end

function HUDManager:AddFeedToKillFeed(unit, unit_name, unit_stats_name, damage_info)
    self._killfeed:AddFeed(unit, unit_name, unit_stats_name, damage_info)
end

function HUDManager:loot_value_updated()
end

function HUDManager:on_ext_inventory_changed()
end

function HUDManager:add_teammate_panel(character_name, player_name, ai, peer_id)
    local i = original.add_teammate_panel(self, character_name, player_name, ai, peer_id)
    if i then
        local synced_cocaine_stacks = managers.player:get_synced_cocaine_stacks(peer_id)
        local maniac = synced_cocaine_stacks and synced_cocaine_stacks.in_use
        self:SetVisibleStatsScreenPanel(i, true)
        self._stats_screen_new:SetAI(i, ai)
        self._stats_screen_new:SetPeerID(i, peer_id)
        self._stats_screen_new:SetCustody(i, false)
        if peer_id then
            local peer = managers.network:session():peer(peer_id)
            self:SetStatsScreenLevel(i, peer:rank(), peer:level())
            local outfit = peer:blackmarket_outfit()
            local perk = outfit.skills.specializations[1] -- String
            local deck = outfit.skills.specializations[2]
            if not maniac and perk then
                deck = tonumber(deck) -- String, converted to integer
                maniac = perk == "14" and deck >= 3 or false
            end
            if perk then
                self:SetStatsScreenPerk(i, tonumber(perk), tonumber(deck))
            end
        end
        if i == HUDManager.PLAYER_PANEL then
            self:SetStatsScreenPing(i, 0, false)
        end
        CSHUD:AddTeammate(character_name, { panel = i, maniac = maniac, AI = ai, peer_id = peer_id })
        self:ChangeManiacVisibility(CSHUD:IsManiacPresent())
        self:RegisterPlayer()
    end
    return i
end

function HUDManager:SetStatsScreenPerk(i, perk, level)
    self._stats_screen_new:SetPerk(i, perk, level)
end

function HUDManager:SetStatsScreenAIPerk(i, loadout)
    self._stats_screen_new:SetAIPerk(i, loadout)
end

function HUDManager:ResetMainPlayer()
    self._stats_screen_new:ResetMainPlayer(HUDManager.PLAYER_PANEL)
end

function HUDManager:RegisterPlayer()
    managers.enemy:on_player_registered()
end

function HUDManager:UnregisterPlayer()
    managers.enemy:on_player_unregistered()
end

function HUDManager:CreateMessagePanel()
    local hud = managers.hud:script(PlayerBase.PLAYER_INFO_HUD_PD2)
    self._message_panel = HUDMessagePanel:new(hud)
end

function HUDManager:ShowBombMessage(time)
    self._message_panel:ShowBombMessage(time)
end

if false then
    function HUDManager:sync_start_assault(assault_number)
        original.sync_start_assault(self, assault_number)
        self._hud_score:SetPlayerScore(assault_number - 1)
    end

    function HUDManager:sync_end_assault(result)
        original.sync_end_assault(self, result)
        local function delay()
            managers.cs_radio:PlaySound("win")
            self._hud_score:SetPlayerScore()
        end
        CSHUD:DelayCall("DelayWinAnnoucement", 0.2, delay)
    end
end

function HUDManager:_create_heist_timer(hud)
    original._create_heist_timer(self, hud)
    self._hud_score = HUDScore:new(tweak_data.levels[Global.game_settings.level_id].hud or {})
end

function HUDManager:set_buff_enabled(buff_name, enabled)
	managers.enemy:set_buff_enabled(buff_name, enabled)
end

if not _G.IS_VR then
    dofile(CSHUD.LuaPath .. "HUDManagerPD2_NonVR.lua")
end