require("lib/managers/menu/ExtendedPanel")
local format_values = CSHUD:GetOption("format_values")
local function sign(v)
	return (v >= 0 and 1) or -1
end

local function round(v, bracket)
    bracket = bracket or 1
	return math.floor(v / bracket + sign(v) * 0.5) * bracket
end

PlayerPanel = class()
function PlayerPanel:init(parent_panel, i, offset)
    local faction, faction_mod = CSHUD:GetPlayerFaction("dark")
    self._equipment = {}
    self._panel = parent_panel:panel({
        name = "panel" .. i,
        x = 0,
        y = 60 + (46 * (i - 1)),
        w = parent_panel:w() - offset,
        h = 44,
        visible = false
    })
    local panel = self._panel
    panel:bitmap({
        name = "bg",
        color = tweak_data.csgo.colors[faction_mod],
        layer = 1,
        x = 70 + offset
    })
    local avatar = panel:bitmap({
        name = "avatar",
        x = 32 + offset,
        y = 2,
        w = 40,
        h = 40,
        texture = tweak_data.csgo.textures.faction.texture,
        texture_rect = tweak_data.csgo.textures.faction[faction].square,
        layer = 4
    })
    local avatar_frame = panel:rect({
        name = "avatar_frame",
        x = 30 + offset,
        y = 0,
        w = 44,
        h = 44,
        color = Color.white,
        layer = 2
    })
    local avatar_bg = panel:rect({
        name = "avatar_bg",
        x = avatar:x(),
        y = avatar:y(),
        w = avatar:w(),
        h = avatar:h(),
        color = Color.black,
        layer = 3,
        visible = false
    })
    local ping = panel:text({
        name = "ping",
        text = "9999",
        layer = 4,
        align = "center",
        vertical = "center",
        font_size = 30,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        x = 0,
        y = panel:y()
    })
    managers.hud:make_fine_text(ping)
    ping:set_center_y(avatar:center_y())
    ping:set_right(avatar:left() - 2)
    local bot_ping = panel:text({
        name = "bot_ping",
        text = "BOT",
        layer = 4,
        align = "center",
        vertical = "center",
        font_size = 30,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        visible = false
    })
    managers.hud:make_fine_text(bot_ping)
    bot_ping:set_center_y(avatar:center_y())
    bot_ping:set_right(avatar:left() - 4)
    local name = panel:text({
        name = "name",
        text = "NAME",
        layer = 5,
        align = "left",
        vertical = "center",
        font_size = 30,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white
    })
    managers.hud:make_fine_text(name)
    name:set_center_y(avatar:center_y())
    name:set_left(avatar:right() + 4)
    local separator_1 = panel:rect({ -- Separator between HP/AP and Player level
        name = "separator_1",
        color = Color.black,
        x = 400 + offset,
        w = 2,
        layer = 10,
        blend_mode = "normal"
    })
    local level = panel:text({
        name = "level",
        text = "100[500]",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 30,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        w = 80
    })
    managers.hud:make_fine_text(level)
    level:set_center_y(avatar:center_y())
    level:set_left(separator_1:right() + 10)
    local separator_2 = panel:rect({ -- Separator between Player level and Player equipment
        name = "separator_2",
        color = Color.black,
        x = 500 + offset,
        w = 2,
        layer = 10,
        blend_mode = "normal"
    })
    local separator_3 = panel:rect({ -- Separator between perk and HP/AP
        name = "separator_3",
        color = Color.black,
        x = 350 + offset,
        w = 2,
        layer = 10,
        blend_mode = "normal"
    })
    local hp = panel:text({
        name = "hp",
        text = format_values == 1 and "100" or "1000",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 20,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        w = 100,
        word_wrap = true
    })
    managers.hud:make_fine_text(hp)
    hp:set_center_y(avatar:center_y())
    hp:set_top(2)
    hp:set_x(350 + offset + ((50 - hp:w()) / 2))
    local ap = panel:text({
        name = "ap",
        text = "100",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 20,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        w = 100,
        word_wrap = true
    })
    managers.hud:make_fine_text(ap)
    ap:set_center_y(avatar:center_y())
    ap:set_top(hp:bottom())
    ap:set_x(350 + offset + ((50 - ap:w()) / 2))
    panel:panel({
        name = "equipment",
        x = 502 + offset,
        w = panel:w() - (502 + offset),
        h = panel:h(),
        visible = true
    })
    local perk = panel:bitmap({
        name = "perk",
        x = separator_3:x() - 32,
        y = 10,
        w = 24,
        h = 24,
        layer = 10,
        visible = true
    })
    local bot_skill = panel:bitmap({
        name = "bot_skill",
        x = separator_3:x() - 28,
        y = 2,
        w = 18,
        h = 18,
        layer = 10,
        visible = false
    })
    local bot_ability = panel:bitmap({
        name = "bot_ability",
        x = separator_3:x() - 28,
        y = 22,
        w = 18,
        h = 18,
        layer = 10,
        visible = false
    })
    local separator_4 = panel:rect({ -- Separator between Player name and perk
        name = "separator_4",
        color = Color.black,
        x = perk:x() - 7,
        w = 2,
        layer = 10,
        blend_mode = "normal"
    })
    local perk_name = panel:text({
        name = "perk_name",
        x = separator_4:right(),
        y = 0,
        w = separator_3:x() - separator_4:right(),
        h = 10,
        text = "",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 10,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
    })
    local perk_progress = panel:text({
        name = "perk_progress",
        x = separator_4:right(),
        y = perk:bottom() - 2,
        w = perk_name:w(),
        h = 10,
        text = "",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 10,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white
    })
    local perk_unknown = panel:text({
        name = "perk_unknown",
        x = perk:x(),
        y = perk:y(),
        w = perk:w(),
        h = perk:h(),
        text = "?",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 24,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        visible = false
    })
end

function PlayerPanel:UpdateAvatar(texture, texture_rect)
    if texture_rect then
        self._panel:child("avatar"):set_image(texture, unpack(texture_rect))
    else
        self._panel:child("avatar"):set_image(texture)
    end
end

function PlayerPanel:SetPerk(perk, level)
    local perk_data = tweak_data.skilltree.specializations[perk]
    if perk_data then
        local tier_data = perk_data[9]
        local guis_catalog = "guis/"
        local texture_rect_x = tier_data.icon_xy and tier_data.icon_xy[1] or 0
	    local texture_rect_y = tier_data.icon_xy and tier_data.icon_xy[2] or 0
        if tier_data.texture_bundle_folder then
            guis_catalog = guis_catalog .. "dlcs/" .. tostring(tier_data.texture_bundle_folder) .. "/"
        end
        local icon_atlas_texture = guis_catalog .. "textures/pd2/specialization/icons_atlas"
        self._panel:child("perk"):set_visible(true)
        self._panel:child("perk_unknown"):set_visible(false)
        self._panel:child("perk"):set_image(icon_atlas_texture, texture_rect_x * 64, texture_rect_y * 64, 64, 64)
        self._panel:child("perk_name"):set_text(managers.localization:text(perk_data.name_id))
    else
        self._panel:child("perk_name"):set_text("Unknown")
        self._panel:child("perk"):set_visible(false)
        self._panel:child("perk_unknown"):set_visible(true)
    end
    self._panel:child("perk_progress"):set_text(level .. "/9")
end

function PlayerPanel:SetAIPerk(loadout)
    if loadout then
        if loadout.skill then
            local icon = tweak_data.upgrades.crew_skill_definitions[loadout.skill]
            if icon then
                local texture, rect = tweak_data.hud_icons:get_icon_data(icon.icon)
                self._panel:child("bot_skill"):set_image(texture, unpack(rect))
                self._panel:child("bot_skill"):set_visible(true)
            else
                self._panel:child("bot_skill"):set_visible(false)
            end
        else
            self._panel:child("bot_skill"):set_visible(false)
        end
        if loadout.ability then
            local icon = tweak_data.upgrades.crew_ability_definitions[loadout.ability]
            if icon then
                local texture, rect = tweak_data.hud_icons:get_icon_data(icon.icon)
                self._panel:child("bot_ability"):set_image(texture, unpack(rect))
                self._panel:child("bot_ability"):set_visible(true)
            else
                self._panel:child("bot_ability"):set_visible(false)
            end
        else
            self._panel:child("bot_ability"):set_visible(false)
        end
    else
        self._panel:child("bot_skill"):set_visible(false)
        self._panel:child("bot_ability"):set_visible(false)
    end
end

function PlayerPanel:SetName(name)
    local player = self._panel
    local panel = player:child("name")
    panel:set_text(name)
    local h = panel:h()
    managers.hud:make_fine_text(panel)
    panel:set_h(h)
    panel:set_font_size(30)
    local w = select(3, panel:text_rect())
    if w > player:child("separator_4"):x() - panel:x() then
        panel:set_font_size(panel:font_size() * (panel:w() / w))
    end
end

if format_values == 1 then
    function PlayerPanel:SetHP(data)
        local panel = self._panel:child("hp")
        local health = data.current / data.total
        panel:set_text(round(health, 0.01) * 100)
    end

    function PlayerPanel:SetAP(data)
        local panel = self._panel:child("ap")
        local armor = data.total ~= 0 and data.current / data.total or 0
        panel:set_text(round(armor, 0.01) * 100)
    end
else
    function PlayerPanel:SetHP(data)
        local panel = self._panel:child("hp")
        panel:set_text(round(data.current, 0.1) * 10)
    end

    function PlayerPanel:SetAP(data)
        local panel = self._panel:child("ap")
        panel:set_text(round(data.current, 0.1) * 10)
    end
end

function PlayerPanel:SetVisible(visible)
    self._panel:set_visible(visible)
end

function PlayerPanel:SetRank(rank, level)
    local panel = self._panel:child("level")
    panel:set_font_size(30)
    local str = level
    if rank > 0 then
        str = str .. "[" .. managers.experience:rank_string(rank) .. "]"
    end
    panel:set_text(str)
    local w = select(3, panel:text_rect())
	if w > panel:w() then
		panel:set_font_size(panel:font_size() * (panel:w() / w))
	end
end

function PlayerPanel:SetAI(ai)
    local panel = self._panel
    if ai then
        panel:child("ping"):set_visible(false)
        panel:child("bot_ping"):set_visible(true)
        panel:child("level"):set_visible(false)
        panel:child("avatar_bg"):set_visible(true)
        panel:child("ap"):set_text("0")
        panel:child("perk"):set_visible(false)
        panel:child("perk_name"):set_visible(false)
        panel:child("perk_progress"):set_visible(false)
    else
        panel:child("ping"):set_visible(true)
        panel:child("bot_ping"):set_visible(false)
        panel:child("level"):set_visible(true)
        panel:child("avatar_bg"):set_visible(false)
        panel:child("perk"):set_visible(true)
        panel:child("perk_name"):set_visible(true)
        panel:child("perk_progress"):set_visible(true)
        panel:child("bot_skill"):set_visible(false)
        panel:child("bot_ability"):set_visible(false)
    end
end

function PlayerPanel:SetPing(ping, packet_loss)
    self._panel:child("ping"):set_text(ping)
    self._panel:child("ping"):set_color(packet_loss and packet_loss > 0 and Color.red or Color.white)
end

function PlayerPanel:SetCustody(in_custody)
    local panel = self._panel
    panel:stop()
    panel:animate(callback(CSAnimation, CSAnimation, "SetAlpha"), in_custody and 0.25 or 1)
end

function PlayerPanel:Remove()
    local panel = self._panel
    panel:child("equipment"):clear()
    panel:set_alpha(1)
    panel:set_visible(false)
    self._equipment = {}
end

function PlayerPanel:AddSpecialEquipment(data)
    local main_panel = self._panel:child("equipment")
    local equipment = self._equipment
    local icon, texture_rect = tweak_data.hud_icons:get_icon_data(data.icon)
    local bitmap = main_panel:bitmap({
        name = data.id,
        layer = 5,
        rotation = 360,
        texture = icon,
        texture_rect = texture_rect,
        w = 16,
        h = 16,
        x = 0,
        y = 0,
        color = self._panel_color
    })
    table.insert(equipment, bitmap)
    self:LayoutSpecialEquipment()
end

function PlayerPanel:RemoveSpecialEquipment(equipment)
    local main_panel = self._panel:child("equipment")
    local equipments = self._equipment
    for i, panel in ipairs(equipments) do
        if panel:name() == equipment then
            table.remove(equipments, i)
            main_panel:remove(panel)
            self:LayoutSpecialEquipment()
            return
        end
    end
end

function PlayerPanel:LayoutSpecialEquipment()
    local main_panel = self._panel:child("equipment")
    local equipments = self._equipment
    local y_x = 0
    for i, panel in ipairs(equipments) do
        panel:set_x(16 * (i - 1))
        if panel:x() > main_panel:w() then
            panel:set_x(16 * y_x)
            panel:set_y(16)
            y_x = y_x + 1
        end
    end
end

function PlayerPanel:ResetMainPlayer()
    local panel = self._panel
    panel:child("equipment"):clear()
    self._equipment = {}
end

function PlayerPanel:SetCallsign(id)
    local alpha = self._panel:child("avatar_frame"):color().a
    self._panel:child("avatar_frame"):set_color((tweak_data.chat_colors[id] or tweak_data.chat_colors[#tweak_data.chat_colors]):with_alpha(alpha))
end

HUDStatsScreenNew = class()
function HUDStatsScreenNew:init()
    self._panel = CSHUD:CreatePanel("stats_panel", 20, true)
    local x = self._panel:w() / 4
    local y = self._panel:h() / 4
    local h = self._panel:h() - (y * 2)
    if HUDManager.BigLobby then
        y = 5
        h = self._panel:h() - 10
    end
    self._tab_panels = {}
    self._tab_panels_info = {}
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._tab_panels_info[i] = {}
    end
    self._stats_panel = self._panel:panel({
        name = "panel",
        x = x,
        y = y,
        w = self._panel:w() - (x * 2),
        h = h
    })
    local blur = self._panel:bitmap({
        texture = "guis/textures/test_blur_df",
		name = "blur_bg",
		layer = -1,
        render_template = "VertexColorTexturedBlur3D",
        halign = "scale",
        valign = "scale",
        x = 0,
        y = 0,
		w = self._panel:w(),
		h = self._panel:h(),
    })
    self._offset = 25
    self:CreatePanelBGAndBlur(self._stats_panel)
    self._panel_color = Color.white
    self:CreateIcons()
    self:CreateText()
    self:CreatePlayerPanels()
    self:CreateMissionItems()
    self:CreateObjectivePanel()
    local tracked = managers.achievment:get_tracked_fill()
    if managers.mutators and managers.mutators:are_mutators_active() then
        self:CreateMutatorsPanel()
    elseif #tracked > 0 then
        self:CreateAchievementTrackedList(tracked)
    end
    if LobbyPlayerInfo then
        self:CreateLPIPanel()
    end
    self:AddListeners()
end

function HUDStatsScreenNew:CreatePanelBGAndBlur(panel)
    local bg = panel:bitmap({
        name = "bg",
        color = Color(128, 52, 48, 45) / 255
    })
    local blur_bg = panel:bitmap({
		texture = "guis/textures/test_blur_df",
		name = "blur_bg",
		layer = -1,
        render_template = "VertexColorTexturedBlur3D",
        halign = "scale",
        valign = "scale",
        x = panel:x(),
        y = panel:y(),
		w = panel:w(),
		h = panel:h(),
    })
    local blur_bg_left = panel:bitmap({
		texture = "guis/textures/test_blur_df",
		name = "blur_bg_left",
		layer = -1,
        render_template = "VertexColorTexturedBlur3D",
        rotation = 180,
        halign = "scale",
        valign = "scale",
        x = panel:x(),
        y = panel:y(),
		w = panel:w(),
		h = panel:h(),
	})
end

function HUDStatsScreenNew:CreateIcons()
    local ping = self._stats_panel:bitmap({
        texture = tweak_data.csgo.textures.stats_screen.texture,
        texture_rect = tweak_data.csgo.textures.stats_screen.ping,
        x = 24,
        y = 35,
        w = 25,
        h = 24,
        layer = 3
    })
end

function HUDStatsScreenNew:CreateText()
    local level = self._stats_panel:text({
        name = "level",
        text = "Level",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 15,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        x = 435 + self._offset,
        y = 45
    })
    managers.hud:make_fine_text(level)
    level:set_x(400 + self._offset + ((100 - level:w()) / 2))
    local equipment = self._stats_panel:text({
        name = "equipment",
        text = "Items",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 15,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        x = 535 + self._offset,
        y = 45
    })
    managers.hud:make_fine_text(equipment)
    equipment:set_x(500 + self._offset + ((90 - equipment:w()) / 2))
    local hp_ap = self._stats_panel:text({
        name = "hp_ap",
        text = "HP\nAP",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 15,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        x = 385 + self._offset,
        y = 30
    })
    managers.hud:make_fine_text(hp_ap)
    hp_ap:set_x(350 + self._offset + ((50 - hp_ap:w()) / 2))
    local perk = self._stats_panel:text({
        name = "perk",
        text = "Perk",
        layer = 5,
        align = "center",
        vertical = "center",
        font_size = 15,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        color = Color.white,
        x = 335 + self._offset,
        y = 45
    })
    managers.hud:make_fine_text(perk)
    perk:set_x(300 + self._offset + ((65 - perk:w()) / 2))
    --local job_data = managers.job:current_job_data()
    local stage_data = managers.job:current_stage_data()
    if stage_data then
        local is_whisper_mode = managers.groupai and managers.groupai:state():whisper_mode()
        local is_level_ghostable = managers.job:is_level_ghostable(managers.job:current_level_id())
        local difficulty = Global.game_settings.difficulty or "easy"
        if is_level_ghostable then
            local ghost_color = is_whisper_mode and Color.white or tweak_data.screen_colors.important_1
            local ghost = self._stats_panel:bitmap({
                texture = "guis/textures/pd2/cn_minighost",
                name = "ghost_icon",
                h = 16,
                w = 16,
                y = 5,
                blend_mode = "add",
                color = ghost_color,
                layer = 90
            })
            ghost:set_x(self._stats_panel:w() - ghost:w() - 5)
            if managers.groupai and managers.groupai:state() and is_whisper_mode then
                managers.groupai:state():add_listener("HUDStatsScreenNew_whisper_mode", {
                    "whisper_mode"
                }, callback(self, self, "on_whisper_mode_changed"))
            end
        end
        local diff = ""
        local diff_color = tweak_data.screen_colors.text
        local localize = true
        if managers.crime_spree and managers.crime_spree:is_active() then
            diff = managers.localization:text("menu_cs_level", {
				level = managers.experience:cash_string(managers.crime_spree:server_spree_level(), "")
            })
            diff_color = tweak_data.screen_colors.risk
            localize = false
        elseif managers.skirmish and managers.skirmish:is_skirmish() then
            diff = "menu_cn_skirmish"
            diff_color = tweak_data.screen_colors.risk
        else
            diff = tweak_data.difficulty_name_ids[difficulty]
            if difficulty ~= "normal" then -- Normal difficulty does not have color set
                diff_color = tweak_data.screen_colors.risk
            end
        end
        local diff_text = self._stats_panel:text({
            name = "diff_text",
            text = localize and managers.localization:to_upper_text(diff) or diff,
            layer = 5,
            align = "center",
            vertical = "center",
            font_size = 15,
            font = tweak_data.menu.pd2_large_font,
            blend_mode = "normal",
            color = diff_color,
            y = 5
        })
        managers.hud:make_fine_text(diff_text)
        if is_level_ghostable then
            diff_text:set_right(self._stats_panel:child("ghost_icon"):left() - 5)
        else
            diff_text:set_x(self._stats_panel:w() - diff_text:w() - 5)
        end
        local heist_name = self._stats_panel:text({
            name = "heist_name",
            text = "Bank Heist",
            layer = 5,
            align = "center",
            vertical = "center",
            font_size = 15,
            font = tweak_data.menu.pd2_large_font,
            blend_mode = "normal",
            color = Color.white,
            y = 5
        })
        local level_data = managers.job:current_level_data()
        if level_data then
            heist_name:set_text(managers.localization:to_upper_text(level_data.name_id))
        end
        managers.hud:make_fine_text(heist_name)
        heist_name:set_right(diff_text:left() - 5)
        local down_modifier_bool = Global.game_settings.no_down or Global.game_settings.one_down
        local down_modifier_color = Color(1, 0.1, 1, 0.5) -- No down
        local down_modifier_text = "no_down_modifier_name"
        if Global.game_settings.one_down and not Global.game_settings.no_down then
            down_modifier_color = tweak_data.screen_colors.one_down
            down_modifier_text = "menu_one_down"
        end
        local down_modifier = self._stats_panel:text({
            name = "down_modifier",
            text = managers.localization:to_upper_text(down_modifier_text),
            layer = 5,
            align = "center",
            vertical = "center",
            font_size = 15,
            font = tweak_data.menu.pd2_large_font,
            blend_mode = "normal",
            color = down_modifier_color,
            y = 20,
            visible = down_modifier_bool and true or false
        })
        managers.hud:make_fine_text(down_modifier)
        down_modifier:set_x(self._stats_panel:w() - down_modifier:w() - 5)
        local job_chain = managers.job:current_job_chain_data()
        local day = managers.job:current_stage()
        local days = job_chain and #job_chain or 0
        local days_text = self._stats_panel:text({
            name = "days",
            text = managers.localization:to_upper_text("hud_days_title", {
                DAY = day,
                DAYS = days
            }),
            layer = 5,
            align = "center",
            vertical = "center",
            font_size = 15,
            font = tweak_data.menu.pd2_large_font,
            blend_mode = "normal",
            y = 20
        })
        managers.hud:make_fine_text(days_text)
        if down_modifier_bool then
            days_text:set_right(down_modifier:left() - 5)
        else
            days_text:set_x(self._stats_panel:w() - days_text:w() - 5)
        end
    end
end

function HUDStatsScreenNew:on_whisper_mode_changed()
    self._stats_panel:child("ghost_icon"):set_color(tweak_data.screen_colors.important_1)
end

function HUDStatsScreenNew:CreatePlayerPanels()
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        self._tab_panels[i] = PlayerPanel:new(self._stats_panel, i, self._offset)
    end
end

function HUDStatsScreenNew:CreateMissionItems()
    self._speaker = self._stats_panel:bitmap({
        name = "speaker",
        texture = tweak_data.csgo.textures.stats_screen.texture,
        texture_rect = tweak_data.csgo.textures.stats_screen.speaker,
        w = 24.5,
        h = 20.75,
        layer = 20,
        color = Color.white,
        x = 5,
        y = 5
    })
    self._music = self._stats_panel:text({
        name = "music",
        text = "<Music>",
        font_size = 15,
        font = tweak_data.menu.pd2_large_font,
        layer = 20
    })
    self:UpdateCurrentMusic()
end

function HUDStatsScreenNew:UpdateCurrentMusic()
    self._music:set_text(managers.music:current_track_string())
    managers.hud:make_fine_text(self._music)
    self._music:set_center_y(self._speaker:center_y())
    self._music:set_left(self._speaker:right() + 5)
end

function HUDStatsScreenNew:CreateObjectivePanel()
    local w = self._stats_panel:left()
    self._objectives = ExtendedPanel:new(self._panel, {
        name = "objectives",
        layer = 30,
        x = 5,
        y = 5,
        w = w - 10,
        h = self._panel:h() - 10
    })
    local job_data = managers.job:current_job_data()
    if job_data and managers.job:current_job_id() == "safehouse" and Global.mission_manager.saved_job_values.playedSafeHouseBefore then
        self._objectives:set_visible(false)
        self.RecreateLeft = function() end
		return
	end
end

function HUDStatsScreenNew:RecreateLeft()
    self._objectives:clear()
    self:CreatePanelBGAndBlur(self._objectives)
    local placer = UiPlacer:new(10, 10, 0, 8)
    placer:add_bottom(self._objectives:fine_text({
		vertical = "top",
		align = "left",
		font_size = tweak_data.hud_stats.objectives_title_size,
		font = tweak_data.hud_stats.objectives_font,
        text = managers.localization:to_upper_text("hud_objective"),
        layer = 80
	}), 16)
    placer:new_row(8)
    local row_w = self._objectives:w() - placer:current_left() * 2
    for i, data in pairs(managers.objectives:get_active_objectives()) do
		placer:add_bottom(self._objectives:fine_text({
			word_wrap = true,
			wrap = true,
			align = "left",
			text = utf8.to_upper(data.text),
			font = tweak_data.hud.medium_font,
			font_size = tweak_data.hud.active_objective_title_font_size,
            w = row_w,
            layer = 81
		}))
		placer:add_bottom(self._objectives:fine_text({
			word_wrap = true,
			wrap = true,
			font_size = 24,
			align = "left",
			text = data.description,
			font = tweak_data.hud_stats.objective_desc_font,
            w = row_w,
            layer = 82
		}), 0)
	end
    local is_whisper_mode = managers.groupai and managers.groupai:state():whisper_mode()
    local medium_font = tweak_data.menu.pd2_medium_font
    local medium_font_size = tweak_data.menu.pd2_medium_font_size
    local loot_panel = ExtendedPanel:new(self._objectives, {
        name = "loot_panel",
		w = self._objectives:w() - 16 - 8
	})
	placer = UiPlacer:new(16, 0, 8, 4)

	if not is_whisper_mode and managers.player:has_category_upgrade("player", "convert_enemies") then
		local minion_text = placer:add_bottom(loot_panel:fine_text({
			keep_w = true,
			text = managers.localization:text("hud_stats_enemies_converted"),
			font = medium_font,
            font_size = medium_font_size,
            layer = 83
		}))

		placer:add_right(nil, 0)

		local minion_texture, minion_rect = tweak_data.hud_icons:get_icon_data("minions_converted")
		local minion_icon = placer:add_left(loot_panel:fit_bitmap({
			w = 17,
			h = 17,
			texture = minion_texture,
            texture_rect = minion_rect,
            layer = 84
		}))

		minion_icon:set_center_y(minion_text:center_y())
		placer:add_left(loot_panel:fine_text({
			text = tostring(managers.player:num_local_minions()),
			font = medium_font,
            font_size = medium_font_size,
            layer = 85
		}), 7)
		placer:new_row()
	end

	if is_whisper_mode then
		local pagers_used = managers.groupai:state():get_nr_successful_alarm_pager_bluffs()
		local max_pagers_data = managers.player:has_category_upgrade("player", "corpse_alarm_pager_bluff") and tweak_data.player.alarm_pager.bluff_success_chance_w_skill or tweak_data.player.alarm_pager.bluff_success_chance
		local max_num_pagers = #max_pagers_data

		for i, chance in ipairs(max_pagers_data) do
			if chance == 0 then
				max_num_pagers = i - 1

				break
			end
		end

		local pagers_text = placer:add_bottom(loot_panel:fine_text({
			keep_w = true,
			text = managers.localization:text("hud_stats_pagers_used"),
			font = medium_font,
            font_size = medium_font_size,
            layer = 83
		}))

		placer:add_right(nil, 0)

		local pagers_texture, pagers_rect = tweak_data.hud_icons:get_icon_data("pagers_used")
		local pagers_icon = placer:add_left(loot_panel:fit_bitmap({
			w = 17,
			h = 17,
			texture = pagers_texture,
            texture_rect = pagers_rect,
            layer = 84
		}))

		pagers_icon:set_center_y(pagers_text:center_y())
		placer:add_left(loot_panel:fine_text({
			text = tostring(pagers_used) .. "/" .. tostring(max_num_pagers),
			font = medium_font,
            font_size = medium_font_size,
            layer = 85
		}), 7)
		placer:new_row()
	end

	local mandatory_bags_data = managers.loot:get_mandatory_bags_data()
	local mandatory_amount = mandatory_bags_data and mandatory_bags_data.amount
	local secured_amount = managers.loot:get_secured_mandatory_bags_amount()
	local bonus_amount = managers.loot:get_secured_bonus_bags_amount()
	local bag_text = placer:add_bottom(loot_panel:fine_text({
		keep_w = true,
		text = managers.localization:text("hud_stats_bags_secured"),
		font = medium_font,
        font_size = medium_font_size,
        layer = 86
	}))

	placer:add_right(nil, 0)

	local bag_texture, bag_rect = tweak_data.hud_icons:get_icon_data("bag_icon")
	local bag_icon = placer:add_left(loot_panel:fit_bitmap({
		w = 16,
		h = 16,
		texture = bag_texture,
        texture_rect = bag_rect,
        layer = 87
	}))

	bag_icon:set_center_y(bag_text:center_y())

	if mandatory_amount and mandatory_amount > 0 then
		local str = bonus_amount > 0 and string.format("%d/%d+%d", secured_amount, mandatory_amount, bonus_amount) or string.format("%d/%d", secured_amount, mandatory_amount)

		placer:add_left(loot_panel:fine_text({
			text = str,
			font = medium_font,
            font_size = medium_font_size,
            layer = 88
		}))
	else
		placer:add_left(loot_panel:fine_text({
			text = tostring(bonus_amount),
			font = medium_font,
            font_size = medium_font_size,
            layer = 89
		}))
	end

	placer:new_row()

	local body_text = placer:add_bottom(loot_panel:fine_text({
		keep_w = true,
		text = managers.localization:to_upper_text("hud_body_bags"),
		font = medium_font,
        font_size = medium_font_size,
        layer = 90
	}))

	placer:add_right(nil, 0)

	local body_texture, body_rect = tweak_data.hud_icons:get_icon_data("equipment_body_bag")
	local body_icon = placer:add_left(loot_panel:fit_bitmap({
		w = 17,
		h = 17,
		texture = body_texture,
        texture_rect = body_rect,
        layer = 91
	}))

	body_icon:set_center_y(body_text:center_y())
	placer:add_left(loot_panel:fine_text({
		text = tostring(managers.player:get_body_bags_amount()),
		font = medium_font,
        font_size = medium_font_size,
        layer = 92
	}), 7)
	placer:new_row()

	local secured_bags_money = managers.experience:cash_string(managers.money:get_secured_mandatory_bags_money() + managers.money:get_secured_bonus_bags_money())

	placer:add_bottom(loot_panel:fine_text({
		keep_w = true,
		text_id = "hud_stats_bags_secured_value",
		font = medium_font,
        font_size = medium_font_size,
        layer = 93
	}), 12)
	placer:add_right(nil, 0)
	placer:add_left(loot_panel:fine_text({
		text = secured_bags_money,
		font = medium_font,
        font_size = medium_font_size,
        layer = 94
	}))
	placer:new_row()

	local instant_cash = managers.experience:cash_string(managers.loot:get_real_total_small_loot_value())

	placer:add_bottom(loot_panel:fine_text({
		keep_w = true,
		text = managers.localization:to_upper_text("hud_instant_cash"),
		font = medium_font,
        font_size = medium_font_size,
        layer = 95
	}))
	placer:add_right(nil, 0)
	placer:add_left(loot_panel:fine_text({
		text = instant_cash,
		font = medium_font,
        font_size = medium_font_size,
        layer = 96
	}))
	loot_panel:set_size(placer:most_rightbottom())
	loot_panel:set_leftbottom(0, self._objectives:h() - 16)
end

local function CreateSharedPanel(self, panel_name)
    local w = self._panel:w() - self._stats_panel:left() - self._stats_panel:w()
    local panel = ExtendedPanel:new(self._panel, {
        name = panel_name,
        x = self._stats_panel:right() + 5,
        y = self._stats_panel:y(),
        w = w - 10,
        h = self._stats_panel:h()
    })
    self["_" .. panel_name] = panel
    self:CreatePanelBGAndBlur(panel)
end

function HUDStatsScreenNew:CreateAchievementTrackedList(tracked)
    local placer = UiPlacer:new(10, 10, 0, 8)
    CreateSharedPanel(self, "tracked_panel")
    self._tracked_items = {}
    placer = UiPlacer:new(0, placer:most().bottom, 0, 0)
    local with_bg = true
    for _, id in pairs(tracked) do
		local t = placer:add_row(HUDTrackedAchievementNew:new(self._tracked_panel, id, with_bg), 0, 0)

		if t._progress and t._progress.update and table.contains({ "realtime", "second" }, t._progress.update) then
			table.insert(self._tracked_items, t)
		end

		with_bg = not with_bg
    end
    if #self._tracked_items > 0 then
        managers.hud:add_updator("HUDStatsScreenNew_AchievementsUpdate", callback(self, self, "Update"))
    end
end

function HUDStatsScreenNew:Update(t, dt)
    for _, v in pairs(self._tracked_items) do
		v:update_progress()
	end
end

function HUDStatsScreenNew:CreateMutatorsPanel()
    local placer = UiPlacer:new(10, 10)
    CreateSharedPanel(self, "mutators_panel")
	placer:add_bottom(self._mutators_panel:fine_text({
		text = managers.localization:to_upper_text("menu_mutators"),
		font = tweak_data.menu.pd2_large_font,
        font_size = tweak_data.hud_stats.objectives_title_size,
        layer = 80
	}))
	for _, active_mutator in ipairs(managers.mutators:active_mutators()) do
		placer:add_row(self._mutators_panel:fine_text({
			text = active_mutator.mutator:name(),
			font = tweak_data.hud_stats.objectives_font,
            font_size = tweak_data.hud_stats.day_description_size,
            layer = 90
		}), 8, 2)
	end
end

-- Code by TdlQ
function HUDStatsScreenNew:CreateLPIPanel()
    if HUDManager.BigLobby or true then
        return
    end
    if LobbyPlayerInfo and LobbyPlayerInfo.settings.show_skills_in_stats_screen then
        self._lpi_panel = self._panel:panel({
            name = "lpi_panel"
        })
        --local r = right_panel:w() - 20
		local placer = UiPlacer:new(10, 10)

		for i = 1, 4 do
			local peer = managers.network:session() and managers.network:session():peer(i)
			placer:add_row(self._right:fine_text({
				name = 'lpi_team_text_name' .. tostring(i),
				align = 'left',
				vertical = 'top',
				blend_mode = 'add',
				font_size = tweak_data.menu.pd2_small_font_size,
				font = tweak_data.menu.pd2_small_font,
				color = tweak_data.chat_colors[i],
				text = peer and peer:name() or '',
			}), 0, 15)

			local outfit = peer and peer:blackmarket_outfit()
			local skills = outfit and outfit.skills
			local perk = skills and skills.specializations

			local skills_txt = ''
			skills = skills and skills.skills
			if skills and #skills >= 15 then
				local ini_len = LobbyPlayerInfo._abbreviation_length_v
				skills_txt = string.format(LobbyPlayerInfo.skills_layouts[#LobbyPlayerInfo.skills_layouts],
					utf8.sub(managers.localization:text('st_menu_mastermind'),  1, ini_len), skills[1],  skills[2],  skills[3],
					utf8.sub(managers.localization:text('st_menu_enforcer'),    1, ini_len), skills[4],  skills[5],  skills[6],
					utf8.sub(managers.localization:text('st_menu_technician'),  1, ini_len), skills[7],  skills[8],  skills[9],
					utf8.sub(managers.localization:text('st_menu_ghost'),       1, ini_len), skills[10], skills[11], skills[12],
					utf8.sub(managers.localization:text('st_menu_hoxton_pack'), 1, ini_len), skills[13], skills[14], skills[15]
				)
			end
			placer:add_row(self._right:fine_text({
				name = 'lpi_team_text_skills' .. tostring(i),
				align = 'left',
				vertical = 'top',
				blend_mode = 'add',
				font_size = tweak_data.menu.pd2_small_font_size - 4,
				font = tweak_data.menu.pd2_small_font,
				color = tweak_data.screen_colors.text,
				text = skills_txt,
			}))

			local perk_txt = ''
			if perk then
				if #perk == 2 then
					perk_txt = LobbyPlayerInfo:GetPerkText(perk[1])
					if tonumber(perk[2]) < 9 then
						perk_txt = perk_txt .. ' (' .. perk[2] .. '/9)'
					end
				else
					perk_txt = 'Unknown perk'
				end
			end
			placer:add_row(self._right:fine_text({
				name = 'lpi_team_text_perk' .. tostring(i),
				align = 'left',
				vertical = 'top',
				blend_mode = 'add',
				font_size = tweak_data.menu.pd2_small_font_size - 4,
				font = tweak_data.menu.pd2_small_font,
				color = tweak_data.screen_colors.text,
				text = perk_txt,
			}))

			--[[local loss_nr = peer and peer:qos().packet_loss or 0
			local ploss_txt = loss_nr == 0 and '' or managers.localization:text('lpi_packet_loss') .. ' (' .. loss_nr .. ')'
			local text_ploss = self._right:fine_text({
				name = 'lpi_team_text_ploss' .. tostring(i),
				text = ploss_txt,
				align = 'right',
				vertical = 'top',
				blend_mode = 'add',
				font_size = tweak_data.menu.pd2_small_font_size - 4,
				font = tweak_data.menu.pd2_small_font,
				color = tweak_data.chat_colors[i],
			})
			placer:add_right(text_ploss)
			text_ploss:set_width(180)
			text_ploss:set_right(r)]]

			if peer and NoMA then
				placer:add_row(self._right:fine_text({
					name = 'lpi_team_text_noma' .. tostring(i),
					align = 'left',
					vertical = 'top',
					blend_mode = 'add',
					font_size = tweak_data.menu.pd2_small_font_size - 4,
					font = tweak_data.menu.pd2_small_font,
					color = tweak_data.screen_colors.text,
					text = NoMA:get_text_info(i),
				}))
			end
		end
    end
end

function HUDStatsScreenNew:AddListeners()
    CSHUD:AddListener(CSHUD.ListenerEvent.UpdateAvatar, callback(self, self, "UpdateAvatar"))
    CSHUD:AddListener(CSHUD.ListenerEvent.SetAI, callback(self, self, "SetAI"))
end

function HUDStatsScreenNew:Show()
    self:RecreateLeft()
    self._panel:show()
end

function HUDStatsScreenNew:Hide()
    self._panel:hide()
end

function HUDStatsScreenNew:SetPerk(i, perk, level)
    self._tab_panels[i]:SetPerk(perk, level)
end

function HUDStatsScreenNew:SetAIPerk(i, loadout)
    self._tab_panels[i]:SetAIPerk(loadout)
end

function HUDStatsScreenNew:UpdateAvatar(i, texture, texture_rect)
    self._tab_panels[i]:UpdateAvatar(texture, texture_rect)
end

function HUDStatsScreenNew:SetName(i, name)
    self._tab_panels[i]:SetName(name)
end

function HUDStatsScreenNew:SetHP(i, data)
    self._tab_panels[i]:SetHP(data)
end

function HUDStatsScreenNew:SetAP(i, data)
    self._tab_panels[i]:SetAP(data)
end

function HUDStatsScreenNew:SetCallsign(i, id)
    self._tab_panels[i]:SetCallsign(id)
end

function HUDStatsScreenNew:SetPanelVisible(i, visible)
    self._tab_panels[i]:SetVisible(visible)
end

function HUDStatsScreenNew:SetRank(i, rank, level)
    self._tab_panels[i]:SetRank(rank, level)
end

function HUDStatsScreenNew:SetAI(i, ai)
    self._tab_panels[i]:SetAI(ai)
    self._tab_panels_info[i].ai = ai
end

function HUDStatsScreenNew:SetPing(i, ping, packet_loss)
    self._tab_panels[i]:SetPing(ping, packet_loss)
end

function HUDStatsScreenNew:SetPeerID(i, peer_id)
    self._tab_panels_info[i].peer_id = peer_id
end

function HUDStatsScreenNew:GetPanelByPeerID(peer_id)
    for i = 1, HUDManager.PLAYER_PANEL, 1 do
        if self._tab_panels_info[i].peer_id and self._tab_panels_info[i].peer_id == peer_id then
            return i
        end
    end
    return nil
end

function HUDStatsScreenNew:SetCustody(i, in_custody)
    self._tab_panels[i]:SetCustody(in_custody)
end

function HUDStatsScreenNew:Remove(i)
    self._tab_panels[i]:Remove()
    self._tab_panels_info[i] = { equipment = {} }
    self:SetPeerID(i, nil)
    self:SetAI(i, nil)
end

function HUDStatsScreenNew:AddSpecialEquipment(i, data)
    self._tab_panels[i]:AddSpecialEquipment(data)
end

function HUDStatsScreenNew:RemoveSpecialEquipment(i, equipment)
    self._tab_panels[i]:RemoveSpecialEquipment(equipment)
end

function HUDStatsScreenNew:ResetMainPlayer(i)
    self._tab_panels[i]:ResetMainPlayer()
end

-- Code copied from HudTrackedAchievement with small differences
HUDTrackedAchievementNew = class(GrowPanel)
function HUDTrackedAchievementNew:init(panel, id, with_bg)
    HUDTrackedAchievementNew.super.init(self, panel, {
        border = 10,
		padding = 4,
		fixed_w = panel:w()
    })
    self._info = managers.achievment:get_info(id)
	self._visual = tweak_data.achievement.visual[id]
	self._progress = self._visual and self._visual.progress
	local placer = self:placer()
	local texture, texture_rect = tweak_data.hud_icons:get_icon_or(self._visual.icon_id, "guis/dlcs/unfinished/textures/placeholder")
	local bitmap = placer:add_bottom(self:bitmap({
		w = 50,
		h = 50,
		texture = texture,
        texture_rect = texture_rect,
        layer = 70
	}))
	local awarded = self._info.awarded

	if not awarded then
		bitmap:set_color(Color.white:with_alpha(0.1))
		self._panel:bitmap({
			texture = "guis/dlcs/trk/textures/pd2/lock",
			w = bitmap:w(),
			h = bitmap:h(),
			x = bitmap:x(),
            y = bitmap:y(),
            layer = 80
		})
	end

	local title_text = managers.localization:text(self._visual.name_id)
	local desc_text = managers.localization:text(self._visual.desc_id)

	placer:add_right(self:fine_text({
		text = title_text,
		font = tweak_data.menu.pd2_medium_font,
        font_size = tweak_data.menu.pd2_medium_font_size,
        layer = 100
	}))

	local desc = self:text({
		wrap = true,
		word_wrap = true,
		text = desc_text,
		font = tweak_data.menu.pd2_tiny_font,
		font_size = tweak_data.menu.pd2_tiny_font_size,
		color = tweak_data.screen_colors.achievement_grey,
        w = self:row_w() - placer:current_left(),
        layer = 99
	})

	self.limit_text_rows(desc, 2)
	placer:add_bottom(self.make_fine_text(desc), 0)

	if self._progress then
		self._bar = placer:add_bottom(TextProgressBar:new(self, {
			w = 235,
			h = 10,
			back_color = Color(255, 60, 60, 65) / 255,
            max = type(self._progress.max) == "function" and self._progress:max() or self._progress.max,
            layer = 90
		}, {
			font_size = 12,
			font = tweak_data.menu.pd2_tiny_font
		}, self._progress:get()))
	end

	if with_bg then
		self:rect({
			layer = -1,
			color = Color.black:with_alpha(0.6)
		})
	end
end

function HUDTrackedAchievementNew:update_progress()
	if self._bar then
		if self._info.awarded then
			local max = type(self._progress.max) == "function" and self._progress:max() or self._progress.max
			self._bar:set_progress(max)
		else
			self._bar:set_progress(self._progress:get())
		end
	end
end