local CSHUD = CSHUD
local original =
{
    on_player_criminal_death = TradeManager.on_player_criminal_death,
    on_AI_criminal_death = TradeManager.on_AI_criminal_death,
    _announce_spawn = TradeManager._announce_spawn,
    sync_teammate_helped_hint = TradeManager.sync_teammate_helped_hint
}
function TradeManager:on_player_criminal_death(criminal_name, respawn_penalty, hostages_killed, skip_netsend)
    local crim = original.on_player_criminal_death(self, criminal_name, respawn_penalty, hostages_killed, skip_netsend)
    if CSHUD._cache.character ~= criminal_name then
        managers.hud:SetCustody(CSHUD:PlaceToCustody(criminal_name), true)
    end
    return crim
end

function TradeManager:on_AI_criminal_death(criminal_name, respawn_penalty, hostages_killed, skip_netsend)
    local crim = original.on_AI_criminal_death(self, criminal_name, respawn_penalty, hostages_killed, skip_netsend)
	if not managers.hud then
		return crim
    end
    if CSHUD._cache.character ~= criminal_name then
        managers.hud:SetCustody(CSHUD:PlaceToCustody(criminal_name), true)
    end
	return crim
end

function TradeManager:_announce_spawn(criminal_name) -- Both host and client
    if criminal_name and CSHUD._cache.character ~= criminal_name then
        managers.hud:SetCustody(CSHUD:RemoveFromCustody(criminal_name), false)
    end
    original._announce_spawn(self, criminal_name)
end

if Network:is_server() then -- Apparently there is a crash when a client revives another player. Field "Revive" is nil even if it is declared above.
    function TradeManager:sync_teammate_helped_hint(helped_unit, helping_unit, hint)
        original.sync_teammate_helped_hint(self, helped_unit, helping_unit, hint)
        if not alive(helped_unit) or not alive(helping_unit) then
            return
        end

        local helped_peer_id = managers.criminals:character_peer_id_by_unit(helped_unit)
        local helping_peer_id = managers.criminals:character_peer_id_by_unit(helping_unit)
        if helped_peer_id and helping_peer_id then
            CSHUD.FunFacts.Revive[helping_peer_id] = CSHUD.FunFacts.Revive[helping_peer_id] or {}
            hint = math.clamp(hint, 2, 3) or 2
            local hint_id = hint == 2 and "helpedup" or "rescued"
            local tbl = CSHUD.FunFacts.Revive[helping_peer_id]
            tbl[hint_id] = tbl[hint_id] or {}
            tbl[hint_id][helped_peer_id] = (tbl[hint_id][helped_peer_id] or 0) + 1
        end
    end
end

if CSHUD:IsKillfeedEnabled() then
    original.on_player_criminal_death_killfeed = TradeManager.on_player_criminal_death
    function TradeManager:on_player_criminal_death(criminal_name, respawn_penalty, hostages_killed, skip_netsend, ...)
        local crim = original.on_player_criminal_death_killfeed(self, criminal_name, respawn_penalty, hostages_killed, skip_netsend, ...)
        local peer_id = managers.criminals:character_peer_id_by_name(criminal_name or "")
        if peer_id then
            local session = managers.network:session()
            local peer = session:peer(peer_id)
            managers.hud:AddFeedToKillFeed(Idstring(criminal_name), "", nil, { variant = "custody", name = peer:name() or "", local_player = peer:id() == session:local_peer():id() })
        end
        return crim
    end
    original.on_AI_criminal_death_killfeed = TradeManager.on_AI_criminal_death
    function TradeManager:on_AI_criminal_death(criminal_name, respawn_penalty, hostages_killed, skip_netsend, ...)
        local crim = original.on_AI_criminal_death_killfeed(self, criminal_name, respawn_penalty, hostages_killed, skip_netsend, ...)
        managers.hud:AddFeedToKillFeed(Idstring(criminal_name), "", nil, { variant = "custody", name = managers.localization:text("menu_" .. criminal_name), local_player = false })
        return crim
    end
end