local function VerifyAndFixWeapon(id, table, default)
    if not table[id] then
        return default or "new_m4" -- Return default weapon if the provided one is not valid
    end
    return id
end

-- These weapons have black background in their textures; no idea why
local weapons_fix =
{
    -- Primaries
    shuno = true,
    -- All weapons in texture bundle folder "osa"
    x_shepheard = true,

    -- Secondaries
    system = true,
    shepheard = true
}

-- These melees have black background in their textures; no idea why
local melee_fix =
{
    meter = true,
    clean = true,
    happy = true,
    aziz = true
}

-- These grenades have black background in their textures; no idea why
local throwable_fix =
{
    damage_control = true,
    tag_team = true,
    pocket_ecm_jammer = true
}

function CSHUD:GetWeaponTextureByWeaponID(w_id, cosmetics)
    local function GetPath(weapon_id, path, dlc)
        if weapons_fix[weapon_id] or dlc == "osa" then
            path = path .. weapon_id
        elseif DB:has(Idstring("texture"), Idstring(path .. "outline/" .. weapon_id)) then
            path = path .. "outline/" .. weapon_id
        elseif DB:has(Idstring("texture"), Idstring("guis/textures/pd2/blackmarket/icons/weapons/outline/" .. weapon_id)) then
            path = "guis/textures/pd2/blackmarket/icons/weapons/outline/" .. weapon_id
        else
            path = path .. weapon_id
        end
        return path
    end

    w_id = VerifyAndFixWeapon(w_id, tweak_data.weapon)

    local cosmetic_id = cosmetics and cosmetics.id

    local guis_catalog = "guis/"
    local bundle_folder = tweak_data.weapon[w_id].texture_bundle_folder

    if bundle_folder then
        guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
    end

    local bitmap_texture = guis_catalog .. "textures/pd2/blackmarket/icons/weapons/"
    local color = nil

    if cosmetic_id then
        local skin_id = tweak_data.blackmarket.weapon_skins[cosmetic_id]
        if self:GetOption("weapon_skin") == 1 then
            if skin_id.is_a_color_skin then
                color = Color(255, 166, 166, 166) / 255 -- Gray
            elseif skin_id.rarity then
                color = tweak_data.economy.rarities[skin_id.rarity].color
            end
            bitmap_texture = GetPath(w_id, bitmap_texture, bundle_folder)
        else
            if skin_id.is_a_color_skin then
                bitmap_texture = GetPath(w_id, bitmap_texture, bundle_folder)
            else
                bitmap_texture = "guis/dlcs/" .. skin_id.texture_bundle_folder .. "/weapon_skins/" .. cosmetic_id
            end
        end
    else
        bitmap_texture = GetPath(w_id, bitmap_texture, bundle_folder)
    end

    return { texture = bitmap_texture, rarity = color }
end

function CSHUD:GetWeaponTexture(factory_id, cosmetics)
    return self:GetWeaponTextureByWeaponID(managers.weapon_factory:get_weapon_id_by_factory_id(factory_id), cosmetics)
end

function CSHUD:GetMeleeTexture(melee_id)
    melee_id = VerifyAndFixWeapon(melee_id, tweak_data.blackmarket.melee_weapons, "fists")
    local guis_catalog = "guis/"
    local bundle_folder = tweak_data.blackmarket.melee_weapons[melee_id].texture_bundle_folder

    if bundle_folder then
        guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
    end

    local bitmap_texture = guis_catalog .. "textures/pd2/blackmarket/icons/melee_weapons/"

    if melee_fix[melee_id] then
        return { texture = bitmap_texture .. melee_id }
    elseif DB:has(Idstring("texture"), Idstring("guis/textures/pd2/blackmarket/icons/melee_weapons/outline/" .. melee_id)) then
        return { texture = "guis/textures/pd2/blackmarket/icons/melee_weapons/outline/" .. melee_id, inverted = true }
    elseif DB:has(Idstring("texture"), Idstring(bitmap_texture .. "outline/" .. melee_id)) then
        return { texture = bitmap_texture .. "outline/" .. melee_id, inverted = true }
    end

    return { texture = bitmap_texture .. melee_id }
end

function CSHUD:GetThrowableTexture(throwable_id)
    if not throwable_id then
        return { texture = nil }
    end
    if throwable_fix[throwable_id] then
        local guis_catalog = "guis/"
        local bundle_folder = tweak_data.blackmarket.projectiles[throwable_id].texture_bundle_folder

        if bundle_folder then
            guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
        end

        return { texture = (guis_catalog .. "textures/pd2/blackmarket/icons/grenades/" .. throwable_id), half = true }
    else
		local prefix = "guis"
		local texture = "/textures/pd2/blackmarket/icons/grenades/outline/" .. throwable_id

		if not DB:has(Idstring("texture"), Idstring(prefix .. texture)) then
			prefix = "guis/dlcs/" .. tweak_data.blackmarket.projectiles[throwable_id].texture_bundle_folder
		end

        return { texture = (prefix .. texture) }
    end
end

function CSHUD:GetCharacterTexture(character_id)
    character_id = CriminalsManager.convert_old_to_new_character_workname(character_id)
    local guis_catalog = "guis/"
    local character_table = tweak_data.blackmarket.characters[character_id] or tweak_data.blackmarket.characters.locked[character_id]
    local bundle_folder = character_table and character_table.texture_bundle_folder

    if bundle_folder then
        guis_catalog = guis_catalog .. "dlcs/" .. tostring(bundle_folder) .. "/"
    end

    local bitmap_texture = guis_catalog .. "textures/pd2/blackmarket/icons/characters/" .. character_id
    return bitmap_texture
end

local friends = nil
local avatars = CSHUD:GetOption("hide_avatar_images")
local function IsFriend(user_id)
    if Steam:logged_on() and user_id then
        friends = friends or Steam:friends()
		for _, friend in ipairs(friends or {}) do
			if friend:id() == user_id then
				return true
			end
		end
	end
	return false
end

function CSHUD:SetAvatar(main_player, peer_id, clbk, try)
    try = try or 0
    self:SetSteamAvatar(main_player, peer_id, clbk, try)
end

function CSHUD:SetSteamAvatar(main_player, peer_id, clbk, try)
    if not managers.network and not managers.network.session and not managers.network:session() and not managers.network:session().peer then
        return
    end
    local peer = peer_id and managers.network:session():peer(peer_id) or nil
    local steam_id = peer and peer:user_id() or Steam:userid()
    local condition = true -- Show All
    if avatars == 2 then -- Hide All (except you)
        condition = false
    elseif avatars == 3 then -- Show Only Friends
        condition = IsFriend(steam_id)
    end
    if steam_id and (main_player or condition) then
        local avatar_size = main_player and Steam.LARGE_AVATAR or 1 -- LARGE_AVATAR: 184x184; else 64x64
        Steam:friend_avatar(avatar_size, steam_id, function(texture)
            local avatar = texture or nil
            local tries = 0
            local steam_texture_obtained = false
            while not avatar or not steam_texture_obtained do
                if tries >= 10 then
                    if try <= 10 then
                        local function Refresh()
                            CSHUD:SetAvatar(main_player, peer_id, clbk, try + 1)
                        end
                        CSHUD:DelayCall("Refresh" .. tostring(main_player) .. tostring(peer_id), 10, Refresh)
                        return
                    end
                    CSHUD:Log("Max tries reached; skipped getting Steam Avatar")
                    break
                end
                Steam:friend_avatar(avatar_size, steam_id, function (texture)
                    local avatar = texture or nil
                    if avatar then
                        clbk(texture)
                        steam_texture_obtained = true
                        return
                    end
                end)
                tries = tries + 1
            end
            if avatar then
                clbk(texture)
            end
        end)
    else
        clbk("guis/textures/pd2_hud_cs/avatar_hidden")
    end
end