local math_lerp = math.lerp
local math_clamp = math.clamp
local math_max = math.max
local math_min = math.min
local math_step = math.step
local math_round = math.round
local math_floor = math.floor
local Color = Color

local DownCounterColor =
{
    [0] = Color.red,
    Color(255, 255, 128, 0) / 255,
    Color.yellow,
    Color(255, 128, 255, 0) / 255,
    Color.green
}

local function Center(o)
    o:set_x((o:parent():w() - o:w()) / 2)
end

local function Center_y(o)
    o:set_y((o:parent():h() - o:h()) / 2)
end

local function round(v, bracket)
    bracket = bracket or 1
    local sign = (v >= 0 and 1) or -1
	return math_floor(v / bracket + sign * 0.5) * bracket
end

local format_values = CSHUD:GetOption("format_values")
local function Clone(type, object, params)
    if not (object or type) then
        return nil
    end
    params = params or {}
    local panel = params.panel or object:parent()
    local new_object
    if type == "bitmap" then
        local final_x = (params.x or object:x()) + (params.move_x or 0)
        local final_y = (params.y or object:y()) + (params.move_y or 0)
        new_object = panel:bitmap({
            name = params.name or object:name(),
            layer = params.layer or object:layer(),
            w = params.w or object:w(),
            h = params.h or object:h(),
            x = final_x,
            y = final_y,
            color = params.color or object:color(),
            texture = params.texture or nil,
            texture_rect = params.texture_rect or nil,
            alpha = params.alpha or object:alpha()
        })
        if params.visible ~= nil then
            new_object:set_visible(params.visible)
        else
            new_object:set_visible(object:visible())
        end
    elseif type == "text" then
        local final_x = (params.x or object:x()) + (params.move_x or 0)
        local final_y = (params.y or object:y()) + (params.move_y or 0)
        new_object = panel:text({
            font = params.font or tweak_data.menu.pd2_large_font,
            font_size = params.font_size or object:font_size(),
            text = params.text or object:text(),
            x = final_x,
            y = final_y,
            w = params.w or object:w(),
            h = params.h or object:h(),
            align = params.align or object:align(),
            alpha = params.alpha or object:alpha(),
            vertical = params.vertical or object:vertical(),
            layer = params.layer or object:layer(),
            color = params.color or object:color(),
            visible = params.visible ~= nil and (params.visible) or (object:visible())
        })
        if params.make_fine_text then
            managers.hud:make_fine_text(new_object)
        end
        if params.visible ~= nil then
            new_object:set_visible(params.visible)
        else
            new_object:set_visible(object:visible())
        end
    elseif type == "rect" then
        local final_x = (params.x or object:x()) + (params.move_x or 0)
        local final_y = (params.y or object:y()) + (params.move_y or 0)
        new_object = panel:rect({
            name = params.name or object:name(),
            layer = params.layer or object:layer(),
            color = params.color or object:color(),
            w = params.w or object:w(),
            h = params.h or object:h(),
            x = final_x,
            y = final_y,
            alpha = params.alpha or object:alpha(),
            visible = params.visible ~= nil and (params.visible) or (object:visible())
        })
        if params.visible ~= nil then
            new_object:set_visible(params.visible)
        else
            new_object:set_visible(object:visible())
        end
    elseif type == "panel" then
        local final_x = (params.x or object:x()) + (params.move_x or 0)
        local final_y = (params.y or object:y()) + (params.move_y or 0)
        local final_w = (params.w or object:w()) + (params.move_w or 0)
        local final_h = (params.h or object:h()) + (params.move_h or 0)
        new_object = panel:panel({
            name = params.name or object:name(),
            x = final_x,
            y = final_y,
            w = final_w,
            h = final_h,
            visible = params.visible ~= nil and (params.visible) or (object:visible())
        })
        if params.visible ~= nil then
            new_object:set_visible(params.visible)
        else
            new_object:set_visible(object:visible())
        end
    end
    return new_object
end

local _f_init = HUDTeammate.init
function HUDTeammate:init(...)
    _f_init(self, ...)
    self._player_panel:hide() -- Hide the original panel, because CS panel is much more better
    self._panel:child("name"):hide()
    self._panel:child("name_bg"):hide()
    self._panel:child("callsign"):hide()
    self._panel:child("callsign_bg"):hide()
    self._radial_health_panel:child("radial_absorb_health_active"):stop() -- Stop animating Vanilla absorbtion
    self._panel_color = CSHUD:GetColorFromTable(CSHUD.settings.hud_color)
    self._primary_color = self._panel_color
    self._secondary_color = self._panel_color
    self._player_color = Color.white

    self._csgo = CSHUD:CreatePanel("csgo", nil, true)

    self._level_id = Global.game_settings.level_id

    self._panel_pos = 1 --CSHUD:GetOption("avatar_time_panel_position")

    self._maniac_present = false

    self._real_id = self._id

    self._custody = false

    self._panel_visible = not (self._level_id == "chill" or HUDManager.BigLobby) -- not (Safehouse or BigLobby mod)

    self:AddPanel()

    self:SetPanelX()

    if self._main_player then
        self:CreateHealthAndArmor()

        self:CreateWeaponsPanel()

        self:CreateUpperHUD()
    end
end

function HUDTeammate:CreateHealthAndArmor()
    local half = self._csgo:w() / 2
    local third = self._csgo:w() / 3
    local third_and_half_of_half = third - (half - third)
    local scale = 1.5
    local bitmap_wh = 20
    local panel_h = 40
    local panel_w = 130 * scale
    -- Health
    self._health_panel = self._csgo:panel({
        name = "health",
        w = 60,
        h = panel_h,
        x = third_and_half_of_half * 2,
        y = self._csgo:h() - panel_h - 10
    })
    self._health_panel:rect({
        name = "bg",
        color = Color.red,
        layer = -10,
        visible = false
    })
    local panel_half = self._health_panel:h() / 2
    self._health_icon = self._health_panel:bitmap({
        name = "health_icon",
        texture = "guis/textures/pd2_hud_cs/csgo/health_icon",
        layer = 11,
        w = bitmap_wh,
        h = bitmap_wh,
        x = 10 * scale,
        y = panel_half - (bitmap_wh / 2),
        color = self._panel_color,
        visible = false
    })
    self._health_text = self._health_panel:text({
        text = format_values == 1 and "000" or "1000",
        name = "health_total",
        align = "center",
        vertical = "center",
        font_size = 30,
        font = tweak_data.csgo.fonts.numbers.font,
        blend_mode = "normal",
        visible = true,
        layer = 2,
        color = self._panel_color
    })
    managers.hud:make_fine_text(self._health_text)
    Center(self._health_text)
    self._health_bar_w = self._health_text:w()
    self._health_bar = self._health_panel:rect({
        name = "health_bar",
        layer = 15,
        w = self._health_bar_w,
        h = 5,
        x = self._health_text:x(),
        color = self._panel_color
    })
    self._health_bar:set_top(self._health_text:bottom() - 4)
    self._health_bar_bg = Clone("rect", self._health_bar, {
        name = "health_bar_bg",
        layer = 14,
        color = Color.red
    })
    self._swan_song_bar = Clone("rect", self._health_bar, {
        name = "swan_song_bar",
        layer = 16,
        color = Color(255, 3, 187, 255) / 255,
        w = 0
    })
    self._health_bar_maniac = Clone("rect", self._health_bar, {
        name = "health_bar_maniac",
        y = self._health_bar:y() + 7,
        visible = false,
        color = Color.blue
    })
    self._health_bar_maniac_bg = Clone("rect", self._health_bar_bg, {
        name = "health_bar_maniac_bg",
        y = self._health_bar_maniac:y(),
        color = Color.white,
        visible = false
    })
    self._health_bar_stoic = Clone("rect", self._health_bar, {
        name = "health_bar_stoic",
        layer = 16,
        visible = false,
        color = tweak_data.csgo.colors.panel_stoic
    })

    -- Armor
    self._armor_panel = Clone("panel", self._health_panel, {
        x = self._health_panel:x(),
        name = "armor"
    })
    self._armor_panel:set_right(self._health_panel:left())
    local armor_icon = self._armor_panel:bitmap({
        name = "icon",
        texture = "guis/textures/pd2_hud_cs/cs2/armor",
        y = 2
    })
    ExtendedPanel.make_bitmap_fit(armor_icon, self._armor_panel:w() / 1.5, self._armor_panel:h() / 1.5)
    Center(armor_icon)
    self._armor_text = Clone("text", self._health_text, {
        panel = self._armor_panel,
        name = "armor_total",
        text = format_values == 1 and "000" or "1000",
        font = tweak_data.csgo.fonts.numbers.font,
        font_size = self._health_text:font_size() - 12,
        move_y = 4.5
    })
    managers.hud:make_fine_text(self._armor_text)
    self._armor_text:set_x(armor_icon:x() - 3)
    self._armor_bar_maniac = Clone("rect", self._health_bar_maniac, {
        panel = self._armor_panel
    })
    self._armor_bar_maniac:set_top(armor_icon:bottom() + 4)
    self._armor_bar_maniac_bg = Clone("rect", self._armor_bar_maniac, {
        panel = self._armor_panel, color = Color.white
    })

    -- Ex-President
    self._health_perk = Clone("panel", self._health_panel, {
        name = "health_perk",
        x = self._armor_panel:x(),
        visible = false
    })
    self._health_perk:set_right(self._armor_panel:left())
    Clone("rect", self._health_panel:child("bg"), { name = "bg", color = Color.green, panel = self._health_perk, visible = false })
    self._health_perk_icon = self._health_perk:bitmap({
        name = "icon",
        layer = 11,
        x = 2,
        y = 2,
        w = 24,
        h = 24,
        texture = "guis/textures/pd2/specialization/icons_atlas",
        texture_rect = { 64 * 3, 64 * 7, 64, 64 } -- Ex-President's First Deck icon
    })
    self._perk_13_text = Clone("text", self._health_text, {
        panel = self._health_perk,
        name = "perk_13_text",
        text = "000",
        x = self._health_perk_icon:right(),
        y = 2,
        font = tweak_data.csgo.fonts.numbers.font,
        make_fine_text = true
    })
    --managers.hud:make_fine_text(self._perk_13_text)
    self._perk_13_bar = Clone("rect", self._health_bar, {
        panel = self._health_perk,
        name = "perk_13_bar",
        y = self._health_perk_icon:top() + self._health_perk_icon:h() + 2
    })
    Clone("rect", self._health_bar_bg, {
        panel = self._health_perk,
        name = "perk_13_bar_bg",
        y = self._perk_13_bar:y(),
        color = tweak_data.csgo.colors.bar_bg
    })

    self.set_custom_radial = function(s, data) -- Overrides Swan Song animation for main player
        local r = data.current / data.total
        s.avatar:child("radial_custom"):set_color(Color(1, r, 1, 1))
        s._swan_song_bar:set_w(self._health_bar_w * r) -- <-- added
    end
end

function HUDTeammate:CreateWeaponsPanel()
    local weapons_w = self._csgo:w() / 3
    local weapons_x = self._csgo:w() - weapons_w
    self._weapons = self._csgo:panel({
        name = "weapons",
        x = CSHUD:ComputeEdgeX(self._csgo:w(), weapons_x),
        y = CSHUD:ComputeEdgeY(self._csgo:h(), 0, CSHUD.Enum.Position.Bottom),
        w = weapons_w,
        rotation = 360
    })
    CSHUD:Log("Panel Y:" .. self._weapons:y(), true)
    local weapons_right_gap = weapons_w - 32
    local weapons_bottom = self._weapons:bottom() - 32

    self._deployable = self._weapons:bitmap({
        name = "deployable",
        w = 64,
        h = 64
    })
    self._deployable:set_bottom(weapons_bottom)
    self._deployable:set_right(weapons_right_gap)

    self._deployable_amount = self._weapons:text({
        name = "deployable_amount",
        align = "center",
        vertical = "center",
        text = "00x",
        font_size = 16,
        font = tweak_data.menu.pd2_large_font,
        color = self._panel_color,
        blend_mode = "normal",
        visible = true,
        layer = 3
    })
    managers.hud:make_fine_text(self._deployable_amount)
    self._deployable_amount:set_bottom(self._deployable:bottom())
    self._deployable_amount:set_left(self._deployable:right() - 7.5)

    self._grenades = Clone("bitmap", self._deployable, { name = "grenades" })
    self._grenades:set_bottom(weapons_bottom)
    self._grenades:set_right(self._deployable:left())

    self._grenades_amount = self._weapons:text({
        name = "grenades_amount",
        text = "00x",
        align = "center",
        vertical = "center",
        font_size = 16,
        font = tweak_data.menu.pd2_large_font,
        color = self._panel_color,
        blend_mode = "normal",
        visible = true,
        layer = 3,
        alpha = 1
    })
    managers.hud:make_fine_text(self._grenades_amount)
    self._grenades_amount:set_bottom(self._grenades:bottom())
    self._grenades_amount:set_left(self._grenades:right() - 22.5)

    self._grenades_refill = self._weapons:text({
        name = "grenades_amount",
        text = "999.9s",
        align = "center",
        vertical = "center",
        font_size = 16,
        font = tweak_data.menu.pd2_large_font,
        color = self._panel_color,
        blend_mode = "normal",
        visible = false,
        layer = 5
    })
    managers.hud:make_fine_text(self._grenades_refill)
    self._grenades_refill:set_top(self._grenades:top())
    self._grenades_refill:set_left(self._grenades:right() - 22.5)

    self._cable_ties = self._weapons:bitmap({
        name = "cable_ties",
        texture = "guis/textures/hud_icons",
        texture_rect = { 384, 96, 48, 48 },
        w = 64,
        h = 64
    })
    self._cable_ties:set_bottom(weapons_bottom)
    self._cable_ties:set_right(self._grenades:left())

    self._cable_ties_amount = self._weapons:text({
        name = "grenades_amount",
        text = "99x",
        align = "center",
        vertical = "center",
        font_size = 16,
        font = tweak_data.menu.pd2_large_font,
        color = self._panel_color,
        blend_mode = "normal",
        visible = true,
        layer = 3
    })
    managers.hud:make_fine_text(self._cable_ties_amount)
    self._cable_ties_amount:set_bottom(self._cable_ties:bottom())
    self._cable_ties_amount:set_left(self._cable_ties:right() - 7.5)

    self._melee = Clone("bitmap", self._grenades, { name = "melee", w = 128 })
    self._melee:set_color(self._panel_color)
    self._melee:set_bottom(self._deployable:top())
    self._melee:set_right(weapons_right_gap)

    self._secondary = Clone("bitmap", self._melee, { name = "secondary" })
    self._secondary:set_bottom(self._melee:top())
    self._secondary:set_right(weapons_right_gap)

    self._primary = Clone("bitmap", self._secondary, { name = "primary" })
    self._primary:set_bottom(self._secondary:top())
    self._primary:set_right(weapons_right_gap)

    self:CreateAmmoPanel()
end

function HUDTeammate:RefreshWeaponTextures()
    local outfit = managers.blackmarket:unpack_outfit_from_string(managers.blackmarket:outfit_string())
    local primary = CSHUD:GetWeaponTexture(outfit.primary.factory_id, outfit.primary.cosmetics)
    local secondary = CSHUD:GetWeaponTexture(outfit.secondary.factory_id, outfit.secondary.cosmetics)
    local third_gap = self._deployable:left()
    if outfit.deployable then
        local deployable = outfit.deployable
        if DB:has(Idstring("texture"), Idstring("guis/textures/pd2/blackmarket/icons/deployables/outline/" .. deployable)) then
            self._deployable:set_image("guis/textures/pd2/blackmarket/icons/deployables/outline/" .. deployable)
        else
            local deployable_tweak_data = tweak_data.blackmarket.deployables[deployable] or {}
            local prefix = "guis/"
            if deployable_tweak_data.texture_bundle_folder then
                prefix = "guis/dlcs/" .. deployable_tweak_data.texture_bundle_folder .. "/"
            end
            local path = prefix .. "textures/pd2/blackmarket/icons/deployables/"
            if DB:has(Idstring("texture"), Idstring(path .. "outline/" .. deployable)) and deployable ~= "grenade_crate" then
                self._deployable:set_image(path .. "outline/" .. deployable)
            elseif DB:has(Idstring("texture"), Idstring(path .. deployable)) then
                self._deployable:set_image(path .. deployable)
            else -- Fallback to question icon
                local texture, _ = tweak_data.hud_icons:get_icon_data("pd2_question")
                self._deployable:set_image(texture)
            end
        end
        self._deployable:set_color(self._panel_color)
        self._deployable_amount:set_text(outfit.deployable_amount .. "x")
    else
        self._deployable:set_visible(false)
        self._deployable_amount:set_visible(false)
    end
    if not (PlayerBase.USE_GRENADES or self._deployable:visible()) then
        third_gap = (self._csgo:w() / 3) - 32
    elseif PlayerBase.USE_GRENADES then
        third_gap = self._grenades:left()
    end
    self._grenades:set_visible(PlayerBase.USE_GRENADES)
    self._cable_ties:set_right(third_gap)
    self._cable_ties_amount:set_left(self._cable_ties:right() - 7.5)
    local grenade = CSHUD:GetThrowableTexture(outfit.grenade)
    self._grenades:set_image(grenade.texture)
    self._grenades:set_h(grenade.half and 32 or 64)
    self._grenades:set_color(self._panel_color)
    if grenade.half then
        self._grenades:set_center(self._deployable:center())
        self._grenades:set_x(self._grenades:x() - 64)
    else
        self._grenades:set_bottom(self._deployable:bottom())
        self._grenades:set_right(self._deployable:left())
    end
    self._grenades_refill:set_top(self._grenades:top())
    self:AlignFourthRow(outfit.deployable, PlayerBase.USE_GRENADES, grenade.half)
    local melee = CSHUD:GetMeleeTexture(outfit.melee_weapon)
    self._melee:set_image(melee.texture)
    if not melee.inverted then
        local width = self._melee:texture_width()
        self._melee:set_texture_rect(width, 0, -width, self._melee:texture_height())
    end
    self._secondary:set_image(secondary.texture)
    self._secondary:set_color(self._secondary_color)
    self._primary:set_image(primary.texture)
    self._primary:set_color(self._primary_color)

    local perk = outfit.skills.specializations[1]-- Why it is in string ?

    local peer_id = managers.network:session():local_peer():id()
    local character = managers.blackmarket:get_real_character(nil, peer_id)
    CSHUD._cache.character = character
    self._real_id = HUDManager.PLAYER_PANEL
    managers.hud:SetVisibleStatsScreenPanel(self._real_id, true)
    managers.hud:SetStatsScreenLevel(self._real_id, managers.experience:current_rank(), managers.experience:current_level())
    managers.hud:SetStatsScreenPerk(self._real_id, tonumber(perk), tonumber(outfit.skills.specializations[2]))
    managers.hud:SetStatsScreenPing(self._real_id, 0)
    managers.hud:RegisterPlayer()
    CSHUD:AddTeammate(character, { panel = self._real_id, maniac = perk == "14", peer_id = peer_id, local_player = true })
    if CSHUD:IsOr(perk, "13", "14", "19") then -- Ex-President, Maniac or Stoic
        local ex_president = perk == "13"
        local maniac = perk == "14"
        local stoic = not (ex_president or maniac)
        self._health_perk:set_visible(ex_president)

        if maniac then
            self:ShowManiac(true)
        end

        if stoic then -- Stoic
            local deck = tonumber(outfit.skills.specializations[2])
            ex_president = deck < 3
            -- Values:
            -- True: You don't have Stoic Perk Deck equipped or 3rd deck in the Stoic Perk Deck is not full (assuming you have Stoic Perk Deck equipped)
            -- False: You have Stoic Perk Deck equipped and 3rd deck in the Stoic Perk Deck is full
            self._armor_panel:set_visible(ex_president)
        end
    end

    self:SetPrimaryWeaponFiremode()
    self:SetSecondaryWeaponFiremode()
end

function HUDTeammate:AlignFourthRow(deployable, grenades, half_grenade)
    if deployable then -- Deployable is present
        if not grenades then -- Grenades are not present
            self._cable_ties:set_right(self._deployable:left())
            self._cable_ties_amount:set_left(self._cable_ties:right() - 7.5)
        end
    else -- Deployable is not present
        if grenades then -- Grenades are present
            self._grenades:set_right(self._deployable:right())
            self._grenades_amount:set_bottom(self._grenades:bottom())
            self._grenades_amount:set_left(self._grenades:right() - 22.5)
            self._grenades_refill:set_top(self._grenades:top())
            self._cable_ties:set_right(self._grenades:left())
        else -- Grenades are not present
            self._cable_ties:set_right(self._deployable:right())
        end
        self._cable_ties_amount:set_left(self._cable_ties:right() - 7.5)
    end
end

function HUDTeammate:SetPrimaryWeaponFiremode()
    self._single_firemode_primary:set_visible(false)
    self._auto_firemode_primary:set_visible(false)
    self._lock_primary:set_alpha(0)
    local primary_weapon = managers.blackmarket:equipped_primary()
    local weapon_tweak_data = tweak_data.weapon[primary_weapon.weapon_id]
    local fire_mode = weapon_tweak_data.FIRE_MODE
    local can_toggle_firemode = weapon_tweak_data.CAN_TOGGLE_FIREMODE
    local locked_to_auto = managers.weapon_factory:has_perk("fire_mode_auto", primary_weapon.factory_id, primary_weapon.blueprint)
    local locked_to_single = managers.weapon_factory:has_perk("fire_mode_single", primary_weapon.factory_id, primary_weapon.blueprint)

    if can_toggle_firemode or not locked_to_auto and fire_mode == "single" then
        self._single_firemode_primary:set_visible(true)
        if not can_toggle_firemode then
            self._lock_primary:set_alpha(1)
        end
    else
        if locked_to_single then
            self._single_firemode_primary:set_visible(true)
        else
            self._auto_firemode_primary:set_visible(true)
        end
        self._lock_primary:set_alpha(1)
    end
end

function HUDTeammate:SetSecondaryWeaponFiremode()
    self._single_firemode_secondary:set_visible(false)
    self._auto_firemode_secondary:set_visible(false)
    self._lock_secondary:set_alpha(0)
    local secondary_weapon = managers.blackmarket:equipped_secondary()
    local weapon_tweak_data = tweak_data.weapon[secondary_weapon.weapon_id]
    local fire_mode = weapon_tweak_data.FIRE_MODE
    local can_toggle_firemode = weapon_tweak_data.CAN_TOGGLE_FIREMODE
    local locked_to_auto = managers.weapon_factory:has_perk("fire_mode_auto", secondary_weapon.factory_id, secondary_weapon.blueprint)
    local locked_to_single = managers.weapon_factory:has_perk("fire_mode_single", secondary_weapon.factory_id, secondary_weapon.blueprint)

    if can_toggle_firemode or not locked_to_auto and fire_mode == "single" then
        self._single_firemode_secondary:set_visible(true)
        if not can_toggle_firemode then
            self._lock_secondary:set_alpha(1)
        end
    else
        if locked_to_single then
            self._single_firemode_secondary:set_visible(true)
        else
            self._auto_firemode_secondary:set_visible(true)
        end
        self._lock_secondary:set_alpha(1)
    end
end

function HUDTeammate:CreateAmmoPanel()
    local half = self._csgo:w() / 2
    local third = self._csgo:w() / 3
    local third_and_half_of_half = third - (half - third)
    local bullet_panel_heigth = 40
    local w = 224
    self._ammo = self._csgo:panel({
        name = "ammo",
        x = self._csgo:w() - (third_and_half_of_half * 2.5),
        y = self._csgo:h() - bullet_panel_heigth - 10,
        w = w,
        h = bullet_panel_heigth,
        rotation = 360
    })

    self._ammo_in_magazine_primary = self._ammo:text({
        text = "000",
        name = "ammo_in_magazine_primary",
        align = "right",
        vertical = "center",
        font_size = 32,
        font = tweak_data.csgo.fonts.numbers.font,
        blend_mode = "normal",
        visible = false,
        layer = 1,
        color = self._panel_color,
        alpha = 1
    })
    managers.hud:make_fine_text(self._ammo_in_magazine_primary)
    Center(self._ammo_in_magazine_primary)
    self._ammo_in_magazine_primary:set_x(2)
    self._ammo_in_magazine_primary:set_y(self._ammo_in_magazine_primary:y() + 2)

    self._ammo_in_magazine_secondary = self._ammo:text({
        text = "000",
        name = "ammo_in_magazine_secondary",
        align = "right",
        vertical = "center",
        font_size = 32,
        font = tweak_data.csgo.fonts.numbers.font,
        blend_mode = "normal",
        visible = false,
        layer = 1,
        color = self._panel_color,
        alpha = 1,
        y = self._ammo_in_magazine_primary:y(),
        x = self._ammo_in_magazine_primary:x()
    })
    managers.hud:make_fine_text(self._ammo_in_magazine_secondary)

    local separator = self._ammo:rect({
        name = "separator",
        align = "center",
        vertical = "center",
        blend_mode = "normal",
        visible = true,
        layer = 1,
        color = self._panel_color,
        alpha = 1,
        w = 1,
        h = 14
    })
    Center_y(separator)
    separator:set_left(self._ammo_in_magazine_secondary:right() + 8)
    separator:set_y(separator:y() - 2)

    self._total_ammo_primary = self._ammo:text({
        text = "0000",
        name = "total_ammo_primary",
        align = "left",
        vertical = "center",
        font_size = 16,
        font = tweak_data.csgo.fonts.numbers.font,
        blend_mode = "normal",
        visible = false,
        layer = 1,
        color = self._panel_color,
        alpha = 1,
        y = separator:y() - 1
    })
    managers.hud:make_fine_text(self._total_ammo_primary)
    self._total_ammo_primary:set_left(separator:right() + 6)

    self._total_ammo_secondary = self._ammo:text({
        text = "0000",
        name = "total_ammo_secondary",
        align = "left",
        vertical = "center",
        font_size = 16,
        font = tweak_data.csgo.fonts.numbers.font,
        blend_mode = "normal",
        visible = false,
        layer = 1,
        color = self._panel_color,
        alpha = 1,
        x = self._total_ammo_primary:x(),
        y = self._total_ammo_primary:y()
    })
    managers.hud:make_fine_text(self._total_ammo_secondary)

    self:CreatePrimaryWeaponFiremode()
    self:CreateSecondaryWeaponFiremode()
end

function HUDTeammate:CreatePrimaryWeaponFiremode()
    self._single_firemode_primary = self._ammo:bitmap({
        name = "single_firemode_primary",
        texture = "guis/textures/pd2_hud_cs/cs2/bullet_single",
        layer = 12,
        color = self._panel_color,
        visible = false,
        x = self._total_ammo_primary:right(),
        w = 16,
        h = 16
    })
    self._single_firemode_primary:set_center_y(self._total_ammo_primary:center_y())
    self._auto_firemode_primary = Clone("bitmap", self._single_firemode_primary, { name = "auto_firemode_primary", texture = "guis/textures/pd2_hud_cs/cs2/bullet_auto" })
    self._lock_primary = self._ammo:bitmap({
        name = "lock_primary",
        texture = "guis/textures/pd2_hud_cs/csgo/padlock",
        layer = 11,
        w = 16,
        h = 16,
        x = self._single_firemode_primary:right(),
        y = self._single_firemode_primary:y(),
        color = self._panel_color,
        visible = false,
        alpha = 1 -- Don't change it to 0, otherwise it does not work
    })
end

function HUDTeammate:CreateSecondaryWeaponFiremode()
    self._single_firemode_secondary = Clone("bitmap", self._single_firemode_primary, { name = "single_firemode_secondary", texture = "guis/textures/pd2_hud_cs/cs2/bullet_single" })
    self._auto_firemode_secondary = Clone("bitmap", self._single_firemode_primary, { name = "auto_firemode_secondary", texture = "guis/textures/pd2_hud_cs/cs2/bullet_auto" })
    self._lock_secondary = Clone("bitmap", self._lock_primary, { name = "lock_secondary", texture = "guis/textures/pd2_hud_cs/csgo/padlock" })
end

function HUDTeammate:set_weapon_firemode(id, firemode, ...)
    local is_secondary = id == 1
    local weapon_type = is_secondary and "_secondary" or "_primary"
    local auto = self["_auto_firemode" .. weapon_type]
    local single = self["_single_firemode" .. weapon_type]
    --local text = self["_firemode_text" .. weapon_type]
    --local text2 = self["_firemode_text_" .. (is_secondary and "primary" or "secondary")] -- Second text; this one hides (to reduce clipping)

    if firemode == "single" then
        single:set_alpha(1)
        auto:set_alpha(0)
    else
        single:set_alpha(0)
        auto:set_alpha(1)
    end
end

function HUDTeammate:CreateUpperHUD()
    local top = self._panel_pos == 1
    local text_faction = self._level_id == "mad" and "ct" or "t"
    local visibility = self._level_id ~= "chill"
    local playing_on = self._csgo:text({
        text = managers.localization:text("hud_team_" .. text_faction),
        name = "playing_on",
        layer = 13,
        align = "center",
        vertical = "center",
        font_size = 24,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        visible = visibility,
        color = tweak_data.csgo.colors[text_faction]
    })
    managers.hud:make_fine_text(playing_on)
    playing_on:set_center(self._csgo:center())
    if top then
        playing_on:set_top(self.avatar:bottom() + 20)
    else
        playing_on:set_bottom(self.avatar:top() - 20)
    end
end

function HUDTeammate:SetRealID(id)
    self._real_id = id
end

function HUDTeammate:SetAvatar(texture, texture_rect)
    if texture_rect then
        self.avatar:child("avatar"):set_image(texture, unpack(texture_rect))
    else
        self.avatar:child("avatar"):set_image(texture)
    end
    managers.hud:UpdateAvatar(self._main_player and HUDManager.PLAYER_PANEL or self._real_id, texture, texture_rect)
end

local white = Color.white
local red = Color.red
function HUDTeammate:SetHPBar(health)
    local bar = self._csgo:child("hp_bar")
    bar:set_w(42 * health)
    bar:set_color(health <= 0.20 and red or white)
end

function HUDTeammate:SetTeammateHPBar(data)
    local health = data.current / data.total
    local bar = self._csgo:child("hp_bar")
    bar:set_w(42 * health)
    bar:set_color(health <= 0.20 and red or white)
end

function HUDTeammate:SetLowerHPBar(health)
    local bar = self._health_bar
    bar:set_w(self._health_bar_w * health)
    bar:set_color(health <= 0.20 and red or self._panel_color)
end

if format_values == 1 then -- Percentage
    function HUDTeammate:set_health(data)
        self._health_data = data
        local health = data.current / data.total
        --self._health_text:set_color(health <= 0.20 and red or self._panel_color)
        self._health_text:set_text((round(health, 0.01) * 100))
        self:SetHPBar(health)
        self._health_bar:set_w(self._health_bar_w * health)
        --self:SetLowerHPBar(health)
        self:update_delayed_damage()
    end

    function HUDTeammate:set_armor(data)
        self._armor_data = data
        local armor = data.total ~= 0 and (data.current / data.total) or 0
        self._armor_text:set_text(round(armor, 0.01) * 100)
        self:update_delayed_damage()
    end
else -- Current
    function HUDTeammate:set_health(data)
        self._health_data = data
        local health = data.current / data.total
        --self._health_text:set_color(health <= 0.20 and red or self._panel_color)
        self._health_text:set_text(round(data.current, 0.1) * 10)
        self:SetHPBar(health)
        self._health_bar:set_w(self._health_bar_w * health)
        --self:SetLowerHPBar(health)
        self:update_delayed_damage()
    end

    function HUDTeammate:set_armor(data)
        self._armor_data = data
        self._armor_text:set_text(round(data.current, 0.1) * 10)
        self:update_delayed_damage()
    end
end

function HUDTeammate:_set_weapon_selected(id, hud_icon)
    local is_secondary = id == 1
    local is_not_secondary = not is_secondary

    self._total_ammo_primary:set_visible(is_not_secondary)
    self._total_ammo_secondary:set_visible(is_secondary)
    self._ammo_in_magazine_primary:set_visible(is_not_secondary)
    self._ammo_in_magazine_secondary:set_visible(is_secondary)

	self._primary:set_alpha(is_secondary and 0.25 or 1)
    self._primary:set_color(is_secondary and self._panel_color or self._primary_color)
    self._secondary:set_alpha(is_secondary and 1 or 0.25)
    self._secondary:set_color(is_secondary and self._secondary_color or self._panel_color)

    self._lock_primary:set_visible(is_not_secondary)
    self._lock_secondary:set_visible(is_secondary)

    self._single_firemode_primary:set_visible(is_not_secondary)
    self._single_firemode_secondary:set_visible(is_secondary)
    self._auto_firemode_primary:set_visible(is_not_secondary)
    self._auto_firemode_secondary:set_visible(is_secondary)
end

function HUDTeammate:set_ammo_amount_by_type(type, max_clip, current_clip, current_left, max, weapon_panel)
    local total_left = current_left - current_clip
    if total_left < 0 then -- Turned off for saws
        total_left = current_left
    end
    self["_total_ammo_" .. type]:set_text(total_left)
    self["_ammo_in_magazine_" .. type]:set_text(current_clip)
end

function HUDTeammate:SetPanelX(i)
    i = (i or self._real_id) - 1
    local x = 420
    local xi = 46
    local hp_bar = self._csgo:child("hp_bar")
    self.avatar:set_x((x + (xi * i)) + CSHUD:ReturnResXDifferenceAbove1920())
    self._stopped:set_center_x(self.avatar:center_x())
    self._carry:set_center_x(self.avatar:center_x())
    if self._panel_pos == 1 then -- Top
        local bottom = self.avatar:bottom()
        self._down_icon:set_top(bottom + 7)
        hp_bar:set_top(bottom + 2)
    else -- Bottom
        local top = self.avatar:top()
        self._down_icon:set_bottom(top - 7)
        hp_bar:set_bottom(top - 2)
    end
    hp_bar:set_x(self.avatar:x() + 1)
    self._down_icon:set_x(self.avatar:x())
    self._down_text:set_x(self._down_icon:x() + 16)
    self._down_text:set_y(self._down_icon:y() + 2.5)
end

function HUDTeammate:AddPanel()
    if self._csgo:child("avatar") then
        self._csgo:remove(self._csgo:child("avatar"))
        self._csgo:remove(self._csgo:child("hp_bar"))
    end
    local cs = tweak_data.csgo
    local faction = CSHUD:IsCT() and "ct" or "t"
    local y = CSHUD:ComputeEdgeY(self._csgo:h(), self._panel_pos == 1 and 2 or (self._csgo:h() - 48))
    local panel_h = 44
    local panel_w = 44
    local avatar = self._csgo:panel({
        name = "avatar",
        w = panel_w,
        h = panel_h,
        y = y,
        visible = self._panel_visible
    })
    avatar:bitmap({
        name = "avatar",
        texture = cs.textures.faction.texture,
        texture_rect = cs.textures.faction[faction].square,
        layer = 12,
        w = 40,
        h = 40,
        x = 2,
        y = 2,
        visible = true
    })
    avatar:bitmap({
        name = "frame",
        texture = cs.textures.frame,
        layer = 12,
        w = avatar:w(),
        h = avatar:h(),
        x = 0,
        y = 0,
        color = Color.white,
        visible = true
    })
    self._csgo:rect({
        name = "hp_bar",
        layer = 13,
        w = panel_w - 2,
        h = 3,
        color = Color.white,
        visible = self._panel_visible
    })
    avatar:rect({
        name = "bg",
        layer = 11,
        w = 40,
        h = 40,
        x = 2,
        y = 2,
        color = Color.black,
        visible = true
    })
    avatar:bitmap({
        name = "ability_meter",
        texture = "guis/textures/pd2_hud_cs/csgo/hud_ability_radial",
        layer = 15,
        render_template = "VertexColorTexturedRadial",
        w = 44,
        h = 44,
        alpha = 1,
        visible = true,
        color = Color(1, 0, 0, 0)
    })
    avatar:bitmap({
        blend_mode = "add",
        name = "ability_icon",
        alpha = 1,
        visible = false,
        layer = 16,
        w = 22,
        h = 22
    })
    avatar:child("ability_icon"):set_center(avatar:child("avatar"):center())
    avatar:text({
        name = "progress_timer",
        text = "45.0s",
        align = "center",
        vertical = "center",
        font_size = 18,
        font = tweak_data.menu.pd2_large_font,
        blend_mode = "normal",
        visible = false,
        color = Color.white,
        layer = 25,
        h = 44,
        w = 44

    })
    managers.hud:make_fine_text(avatar:child("progress_timer"))
    avatar:child("progress_timer"):set_center(avatar:child("avatar"):center())
    self:CreateConditionTimer(avatar)
    avatar:bitmap({
        name = "radial_custom",
        texture = "guis/textures/pd2_hud_cs/csgo/hud_swan_song",
        visible = true,
        alpha = 1,
        render_template = "VertexColorTexturedRadial",
        layer = 25,
        color = Color(1, 0, 0, 0),
        w = 44,
        h = 44
    })
    self.avatar = avatar

    -- AI Stopped Icon
    self._stopped = self._csgo:bitmap({
        name = "stopped",
		texture = tweak_data.hud_icons.ai_stopped.texture,
        texture_rect = tweak_data.hud_icons.ai_stopped.texture_rect,
        w = 16,
        h = 32,
        visible = false
    })
    self._stopped:set_y(self._panel_pos == 1 and avatar:bottom() + 2 or avatar:top() - 50)
    self._stopped_offset = self._stopped:y()
    self._stopped_offset_moved = self._stopped_offset + (self._panel_pos == 1 and 31 or -31)

    -- Carry Icon
    self._carry = self._csgo:bitmap({
        name = "carry",
		texture = "guis/textures/pd2/hud_tabs",
        texture_rect = { 32, 33, 32, 31 },
        w = 32,
        h = 31,
        visible = false
    })
    self._carry:set_y(self._panel_pos == 1 and avatar:bottom() + 16 or avatar:top() - 48)

    -- Down Counter
    self._down_icon = self._csgo:bitmap({
        name = "down_icon",
        texture = "guis/textures/pd2/skilltree_2/icons_atlas_2",
        texture_rect = { 400, 160, 80, 80 },
        w = 16,
        h = 16,
        visible = self._main_player and self._panel_visible
    })
    self._down_text = self._csgo:text({
        blend_mode = "normal",
        name = "down_text",
        align = "center",
        vertical = "center",
        text = "3x",
        w = 16,
        h = 16,
        visible = self._down_icon:visible(),
        color = DownCounterColor[3],
        font = tweak_data.hud_players.ammo_font,
        font_size = 14
    })
    managers.hud:make_fine_text(self._down_text)
end

function HUDTeammate:SetDefaultAvatar() -- Set only if you drop-in => to fix duplicate Steam Avatar
    local faction = CSHUD:IsCT() and "ct" or "t"
    local texture = tweak_data.csgo.textures.faction
    self:SetAvatar(texture.texture, texture[faction].square)
end

function HUDTeammate:CreateConditionTimer(avatar)
    avatar:bitmap({
        blend_mode = "add",
        name = "condition_icon",
        alpha = 1,
        visible = false,
        layer = 16,
        w = 40,
        h = 40,
        x = 2,
        y = 2,
        color = Color.white
    })
    avatar:text({
		vertical = "center",
		name = "condition_timer",
		align = "center",
		text = "000",
		visible = false,
		layer = 20,
		color = Color.white,
		font_size = tweak_data.hud_players.timer_size,
		font = tweak_data.hud_players.timer_font
    })
    avatar:child("condition_timer"):set_center(avatar:child("condition_icon"):center())
end

function HUDTeammate:set_revives_amount(revive_amount)
	if revive_amount then
        self._down_text:set_text(tostring(math_max(revive_amount - 1, 0)) .. "x")
        self._down_text:set_color(DownCounterColor[revive_amount - 1] or Color.white)
	end
end

function HUDTeammate:set_state(state)
    if state == "player" then
        self:SetDownCounterVisibility(true)
    end
end

function HUDTeammate:SetDownCounterVisibility(visibility)
    if not self._panel_visible then
        return
    end
    self._down_icon:set_visible(visibility)
    self._down_text:set_visible(visibility)
end

function HUDTeammate:set_custom_radial(data)
	local r = data.current / data.total
	self.avatar:child("radial_custom"):set_color(Color(1, r, 1, 1))
end

function HUDTeammate:set_ability_radial(data)
    local progress = data.current / data.total
    self.avatar:child("ability_meter"):set_color(Color(1, progress, 1, 1))
    self.avatar:child("ability_icon"):set_visible(progress > 0)
end

function HUDTeammate:activate_ability_radial(time_left, time_total)
    time_total = time_total or time_left
    local progress_start = time_left / time_total
    local meter = self.avatar:child("ability_meter")
    local icon = self.avatar:child("ability_icon")

    local function anim(o)
        icon:set_visible(true)
        over(time_left, function(p)
            local progress = progress_start * math_lerp(1, 0, p)

            meter:set_color(Color(1, progress, 1, 1))
        end)
        icon:set_visible(false)
    end

    meter:stop()
    meter:animate(anim)

    if self._main_player then
		local current_time = managers.game_play_central:get_heist_timer()
		local end_time = current_time + time_left

		managers.network:session():send_to_peers("sync_ability_hud", end_time, time_total)
	end
end

function HUDTeammate:set_ability_icon(icon)
    local texture, texture_rect = tweak_data.hud_icons:get_icon_data(icon, {
        0,
        0,
        32,
        32
    })

    self.avatar:child("ability_icon"):set_image(texture, unpack(texture_rect))
end

local avatar_state =
{
    mugshot_normal = true,
    mugshot_in_custody = true
}
function HUDTeammate:set_condition(icon_data, text)
    local panel = self.avatar
    local condition_icon = panel:child("condition_icon")
    local avatar = panel:child("avatar")
	if avatar_state[icon_data] then
        condition_icon:set_visible(false)
        avatar:set_visible(true)
    else
        avatar:set_visible(false)
        condition_icon:set_visible(true)

		local icon, texture_rect = tweak_data.hud_icons:get_icon_data(icon_data)

        condition_icon:set_image(icon, texture_rect[1], texture_rect[2], texture_rect[3], texture_rect[4])
    end
end

-- Figure out better solution
-- EquipmentsTweakData.lua has icon definition. Maybe change there ?
local equipment_name_fix =
{
    sentry = "sentry_gun",
    sentry_silent = "sentry_gun_silent"
}
function HUDTeammate:set_deployable_equipment(data)
    if self._main_player then
        local deployable, _ = data.icon:gsub("equipment_", "")
        local icon = equipment_name_fix[deployable] or deployable
        self._deployable:set_image("guis/textures/pd2/blackmarket/icons/deployables/outline/" .. icon)
        self:set_deployable_equipment_amount(1, data)
    end
end

function HUDTeammate:set_deployable_equipment_amount(index, data)
    if self._main_player then
        local condition = data.amount == 0
        local alpha = condition and 0.25 or 1
        self._deployable_amount:set_text(data.amount .. "x")
        self._deployable_amount:set_visible(not condition)
        self._deployable:stop()
        self._deployable:animate(callback(CSAnimation, CSAnimation, "SetAlpha"), alpha)
    end
end

function HUDTeammate:set_grenades(data)
	if not PlayerBase.USE_GRENADES then
		return
	end

    if self._main_player then
        self:set_grenades_amount(data)
    end
end

function HUDTeammate:set_grenades_amount(data)
	if not PlayerBase.USE_GRENADES then
		return
    end

    local condition = data.amount ~= 0
    local alpha = condition and 1 or 0.25

    self._grenades:stop()
    self._grenades:animate(callback(CSAnimation, CSAnimation, "SetAlpha"), alpha)
    self._grenades_amount:set_text(data.amount .. "x")
    self._grenades_amount:set_visible(condition)
end

function HUDTeammate:add_panel()
    if not self._wait_panel:visible() then
        self:ShowPanel()
    end
    if self._main_player then
        if self._team_showed then
            return
        end
        local function HideTeam()
            self._csgo:child("playing_on"):animate(function(o)
                local t = 1
                while t > 0 do
                    t = t - (coroutine.yield() / 2)
                    o:set_alpha(t)
                end
                o:set_alpha(0)
            end)
            self._team_showed = true
        end
        CSHUD:DelayCall("HideTeam", 5, HideTeam)
        if managers.enemy then
            managers.enemy._csgo:show()
        end
    else
        managers.hud:CustodyTeammatePanels_SetPanelVisible(self._real_id, true)
    end
end

function HUDTeammate:remove_panel(weapons_panel)
    self:HidePanel()
    self:SetCustody(false)

    local teammate_panel = self._panel

    if self._main_player then
        managers.hud:ResetMainPlayer()
        local special_equipment = self._special_equipment
        while special_equipment[1] do
            teammate_panel:remove(table.remove(special_equipment))
        end
    end

	self:set_condition("mugshot_normal")

	weapons_panel = weapons_panel or self._player_panel:child("weapons_panel")

	weapons_panel:child("secondary_weapon_panel"):set_visible(false)
	weapons_panel:child("primary_weapon_panel"):set_visible(false)
	self._deployable_equipment_panel:child("equipment"):set_visible(false)
	self._deployable_equipment_panel:child("amount"):set_visible(false)
	self._cable_ties_panel:child("cable_ties"):set_visible(false)
    self._cable_ties_panel:child("amount"):set_visible(false)
    self._carry:set_visible(false)
	self:set_cheater(false)
	self:set_info_meter({
		total = 0,
		current = 0,
		max = 1
	})
	self:stop_timer()
	self:teammate_progress(false, false, false, false)

	self._peer_id = nil
    self._ai = nil

    managers.hud:CustodyTeammatePanels_SetAI(self._real_id, false)

    self._stopped:set_visible(false)

    self:SetDownCounterVisibility(false)

    self:UpdateIcons()

    if not self._main_player then
        managers.hud:CustodyTeammatePanels_SetPanelVisible(self._real_id, false)
    end
end

function HUDTeammate:HidePanel()
    self._csgo:set_visible(false)
end

function HUDTeammate:ShowPanel()
    self._csgo:set_visible(true)
end

function HUDTeammate:start_timer(time)
	self._timer_paused = false
    self._timer = time

    local timer = self.avatar:child("condition_timer")

	timer:set_font_size(tweak_data.hud_players.timer_size)
	timer:set_color(Color.white)
	timer:stop()
	timer:set_visible(true)
	timer:animate(callback(self, self, "_animate_timer"))
end

function HUDTeammate:set_pause_timer(pause)
	self._timer_paused = pause
end

function HUDTeammate:stop_timer()
	if not alive(self.avatar) then
		return
	end

    local timer = self.avatar:child("condition_timer")
	timer:set_visible(false)
	timer:stop()
end

function HUDTeammate:is_timer_running()
	return self.avatar:child("condition_timer"):visible()
end

function HUDTeammate:_animate_timer(o)
    local rounded_timer = math_round(self._timer)

	while self._timer >= 0 do
		local dt = coroutine.yield()

		if not self._timer_paused then
			self._timer = self._timer - dt
			local text = self._timer < 0 and "00" or (math_round(self._timer) < 10 and "0" or "") .. math_round(self._timer)

			o:set_text(text)

			if math_round(self._timer) < rounded_timer then
				rounded_timer = math_round(self._timer)

				if rounded_timer < 11 then
					o:animate(callback(self, self, "_animate_timer_flash"))
				end
			end
		end
	end
end

function HUDTeammate:_animate_timer_flash(o)
	local t = 0

	while t < 0.5 do
		t = t + coroutine.yield()
		local n = 1 - math.sin(t * 180)
		local r = math_lerp(1, 1, n)
		local g = math_lerp(0, 0.8, n)
		local b = math_lerp(0, 0.2, n)

		o:set_color(Color(r, g, b))
		o:set_font_size(math_lerp(tweak_data.hud_players.timer_size, tweak_data.hud_players.timer_flash_size, n))
	end

	o:set_font_size(30)
end

function HUDTeammate:MoveHUDElementsBot()
    self._carry:set_y(self._panel_pos == 1 and self.avatar:bottom() or self.avatar:top() - 32)
    self._stopped:set_y(self._carry:y())
    self._stopped_offset = self._stopped:y()
    self._stopped_offset_moved = self._stopped_offset + (self._panel_pos == 1 and 31 or -31)
end

function HUDTeammate:MoveHUDElementsPlayer()
    self._carry:set_y(self._panel_pos == 1 and self.avatar:bottom() + 16 or self.avatar:top() - 48)
end

function HUDTeammate:SetBotAvatar()
    local function DelayAIAvatar()
        local characters = managers.criminals._characters or {}
        for _, char in pairs(characters) do
            if char.name and char.data.ai and char.data.panel_id and char.data.panel_id == self._real_id then
                if CSHUD:GetOption("bot_avatar") then
                    local bitmap_texture = CSHUD:GetCharacterTexture(char.name)
                    self.avatar:child("avatar"):set_image(bitmap_texture)
                    managers.hud:UpdateAvatar(self._real_id, bitmap_texture)
                end
                self:SetHPBar(1)
                managers.hud:CustodyTeammatePanels_SetAI(self._real_id, true)
                break
            end
        end
    end
    CSHUD:DelayCall("DelayAIAvatar" .. self._real_id, 1, DelayAIAvatar)
end

local _f_set_ai = HUDTeammate.set_ai
function HUDTeammate:set_ai(ai)
    _f_set_ai(self, ai)
    if ai then
        self:SetBotAvatar()
        self:MoveHUDElementsBot()
    end
end

local _f_set_peer_id = HUDTeammate.set_peer_id
function HUDTeammate:set_peer_id(peer_id)
    _f_set_peer_id(self, peer_id)
    if peer_id then
        local function DelayTeammateAvatar()
            CSHUD:SetAvatar(nil, peer_id, callback(self, self, "SetAvatar"))
            managers.hud:CustodyTeammatePanels_SetWeapons(self._real_id, peer_id)
        end
        CSHUD:DelayCall("DelayTeammateAvatar" .. self._real_id, 1, DelayTeammateAvatar)
        self:MoveHUDElementsPlayer()
    end
end

function HUDTeammate:teammate_progress(enabled, tweak_data_id, timer, success)
	self.avatar:child("progress_timer"):stop()
    self.avatar:child("progress_timer"):set_visible(enabled)
    self.avatar:child("avatar"):set_visible(not enabled)

	if enabled then
		self.avatar:child("progress_timer"):animate(self._animate_label_interact, timer)
	end
end

function HUDTeammate._animate_label_interact(o, timer)
    local t = 0

    o:set_text(timer .. "s")

	while t <= timer do
        t = t + coroutine.yield()
        o:set_text(round(timer - t, 0.1) .. "s")
	end

    o:set_text("0s")
end

CSHUD:Hook(HUDTeammate, "set_callsign", function(self, id)
    local alpha = self._panel:child("callsign"):color().a
    self._player_color = (tweak_data.chat_colors[id] or tweak_data.chat_colors[#tweak_data.chat_colors]):with_alpha(alpha)
    self.avatar:child("frame"):set_color(self._player_color)
end)

function HUDTeammate:set_cable_tie(data)
	self:set_cable_ties_amount(data.amount)

	return nil
end

function HUDTeammate:set_cable_ties_amount(amount)
    local condition = amount == 0
    local alpha = condition and 0.25 or 1

	if amount == -1 then
		self._cable_ties_amount:set_text("--")
    else
        self._cable_ties_amount:set_text(amount .. "x")
    end
    self._cable_ties_amount:set_visible(not condition)

    self._cable_ties:stop()
	self._cable_ties:animate(callback(CSAnimation, CSAnimation, "SetAlpha"), alpha)
end

function HUDTeammate:set_stored_health_max(stored_health_ratio)
    self._stored_health_max = stored_health_ratio
end

function HUDTeammate:set_stored_health(stored_health_ratio)
    local r = math_min(stored_health_ratio, 1)
    self._perk_13_bar:set_w(math_lerp(0, self._health_bar_w, stored_health_ratio / self._stored_health_max))
    self._perk_13_text:set_text(round(r, 0.01) * 100)
end

function HUDTeammate:set_ai_stopped(stopped)
    if self._ai then
        self._stopped:set_visible(stopped)
    end
end

function HUDTeammate:set_carry_info()
    self._carry:set_visible(true)
    self:UpdateIcons(true)
end

function HUDTeammate:remove_carry_info()
    self._carry:set_visible(false)
    self:UpdateIcons()
end

function HUDTeammate:UpdateIcons(carry)
    if carry then
        self._stopped:set_y(self._stopped_offset_moved)
    else
        self._stopped:set_y(self._stopped_offset)
    end
end

function HUDTeammate:set_cheater(state)
	self.avatar:child("frame"):set_color(state and tweak_data.screen_colors.pro_color or self._player_color)
end

function HUDTeammate:update_delayed_damage()
    local damage = self._delayed_damage or 0
    local health_bar = self._health_bar
    local health_bar_stoic = self._health_bar_stoic
    local armor_current = self._armor_data.current
    local health_max = self._health_data.total
	local health_current = self._health_data.current
    local health_ratio = health_bar:w() / 80
    local armor_damage = damage < armor_current and damage or armor_current
    damage = damage - armor_damage
    local health_damage = health_current > damage and damage or health_current
    local health_damage_ratio = health_damage / health_max
    local health_stoic_w = 80 * health_damage_ratio

    health_bar_stoic:set_visible(health_damage_ratio > 0)
    health_bar_stoic:set_w(health_stoic_w)
    health_bar_stoic:set_x(math_max((health_bar:x() + (80 * health_ratio) - health_stoic_w), 80))
end

function HUDTeammate:set_grenade_cooldown(data)
	if not PlayerBase.USE_GRENADES then
		return
	end

    local grenades_timer = self._grenades_refill
	local end_time = data and data.end_time
	local duration = data and data.duration

	if not (end_time and duration) then
        grenades_timer:stop()
        grenades_timer:set_text("")

		return
	end

    local function animate_radial(o)
        o:set_visible(true)
		repeat
			local now = managers.game_play_central:get_heist_timer()
            local time_left = end_time - now
            local r = time_left >= 100 and 1 or 0.1

            o:set_text(round(time_left, r) .. "s")
			coroutine.yield()
		until time_left <= 0

        o:set_text("0s")
		o:set_visible(false)
	end

	grenades_timer:stop()
	grenades_timer:animate(animate_radial)

	if self._main_player then
		managers.network:session():send_to_peers("sync_grenades_cooldown", end_time, duration)
	end
end

function HUDTeammate:animate_grenade_flash()
    local grenade = self._grenades

    local function anim(o)
        local start_a = o:alpha()
        local t
        for i = 1, 3, 1 do
            t = 0.5
            while t > 0 do
                t = t - coroutine.yield()
                o:set_alpha(math.sin(t * 180))
            end
            t = 0
            o:set_alpha(0)
            while t < 0.5 do
                t = t + coroutine.yield()
                o:set_alpha(math.sin(t * 180))
            end
            o:set_alpha(1)
        end
        -- When the blinking ended; return back to original alpha
        if start_a == 1 then
            return
        end
    end

    grenade:stop()
    --grenade:animate(anim)
end

function HUDTeammate:add_special_equipment(data)
	local main_panel = self._csgo
	local special_equipment = self._special_equipment
	local equipment_panel = main_panel:panel({
		y = self._primary:top() - 32,
		layer = 2,
        name = data.id,
        w = 32,
        h = 32
    })
    local icon, texture_rect = tweak_data.hud_icons:get_icon_data(data.icon)

	local bitmap = equipment_panel:bitmap({
		name = "bitmap",
		layer = 0,
		rotation = 360,
		texture = icon,
		color = self._panel_color,
		texture_rect = texture_rect,
		w = equipment_panel:w(),
		h = equipment_panel:h()
	})
	local w = main_panel:w() - 64

    equipment_panel:set_x(w - (equipment_panel:w() + 0) * #special_equipment)
	table.insert(special_equipment, equipment_panel)

	local amount, amount_bg = nil, nil

	if data.amount then
		amount_bg = equipment_panel:child("amount_bg") or equipment_panel:bitmap({
			texture = "guis/textures/pd2/equip_count",
			name = "amount_bg",
			rotation = 360,
			layer = 2,
			color = Color.white
		})
		amount = equipment_panel:child("amount") or equipment_panel:text({
			name = "amount",
			vertical = "center",
			font_size = 12,
			align = "center",
			font = "fonts/font_small_noshadow_mf",
			rotation = 360,
			layer = 3,
			text = tostring(data.amount),
			color = Color.black,
			w = equipment_panel:w(),
			h = equipment_panel:h()
		})

		amount_bg:set_center(bitmap:center())
		amount_bg:move(7, 7)
		amount_bg:set_visible(data.amount > 1)
		amount:set_center(amount_bg:center())
		amount:set_visible(data.amount > 1)
    end

    equipment_panel:stop()
    equipment_panel:animate(callback(CSAnimation, CSAnimation, "Flash"), 3)

	self:layout_special_equipments()
end

function HUDTeammate:remove_special_equipment(equipment)
	local teammate_panel = self._csgo
	local special_equipment = self._special_equipment

	for i, panel in ipairs(special_equipment) do
        if panel:name() == equipment then
            panel:stop() -- Stop the animation, if any is in progress
            table.remove(special_equipment, i)

			teammate_panel:remove(panel)
			self:layout_special_equipments()

			return
		end
	end
end

function HUDTeammate:layout_special_equipments()
    local main_panel = self._csgo
    local special_equipment = self._special_equipment
    local panel_w = main_panel:w() - 64
    for i, panel in ipairs(special_equipment) do
        panel:set_x(panel_w - (panel:w() + 0) * (i - 1))
    end
end

function HUDTeammate:SetCustody(in_custody)
    self._custody = in_custody
    if not HUDManager.BigLobby then
        local panel = self._csgo:child("avatar")
        panel:stop()
        panel:animate(callback(CSAnimation, CSAnimation, "SetAlpha"), in_custody and 0.25 or 1)
    end
    if self._main_player then
        self._health_panel:set_visible(not in_custody)
        self._armor_panel:set_visible(not in_custody)
        self._ammo:set_visible(not in_custody)
    end
end

function HUDTeammate:ShowManiac(visibility)
    if visibility then
        if self._maniac_present then
            return
        end
        self._health_bar_maniac:set_visible(true)
        self._health_bar_maniac:animate(callback(self, self, "animate_update_absorb_active"))
        self._health_bar_maniac_bg:set_visible(true)
        self._armor_bar_maniac:set_visible(true)
        self._armor_bar_maniac_bg:set_visible(true)
        self._maniac_present = true
    else
        if not self._maniac_present then
            return
        end
        self._health_bar_maniac:set_visible(false)
        self._health_bar_maniac:stop()
        self._armor_bar_maniac:set_visible(false)
        self._armor_bar_maniac_bg:set_visible(false)
        self._maniac_present = false
    end
end

function HUDTeammate:_animate_update_absorb(o, radial_absorb_shield_name, radial_absorb_health_name, var_name, blink)
	repeat
		coroutine.yield()
	until alive(self._panel) and self[var_name] and self._armor_data and self._health_data

    local teammate_panel = self._csgo
    local armor = self._armor_bar_maniac
    local hp = self._health_bar_maniac
    armor:set_w(0)
    hp:set_w(0)
	--[[local radial_health_panel = self._radial_health_panel
	local radial_shield = radial_health_panel:child("radial_shield")
	local radial_health = radial_health_panel:child("radial_health")
	local radial_absorb_shield = radial_health_panel:child(radial_absorb_shield_name)
	local radial_absorb_health = radial_health_panel:child(radial_absorb_health_name)
	local radial_shield_rot = radial_shield:color().r
	local radial_health_rot = radial_health:color().r

	radial_absorb_shield:set_rotation((1 - radial_shield_rot) * 360)
	radial_absorb_health:set_rotation((1 - radial_health_rot) * 360)]]

	local current_absorb = 0
	local current_shield, current_health = nil, nil
	local step_speed = 1
	local lerp_speed = 1
	local dt, update_absorb = nil, nil
    local bar_w = self._health_bar_w

	while alive(teammate_panel) do
		dt = coroutine.yield()

		if self[var_name] and self._armor_data and self._health_data then
			update_absorb = false
			current_shield = self._armor_data.current
			current_health = self._health_data.current

			if current_absorb ~= self[var_name] then
				current_absorb = math_lerp(current_absorb, self[var_name], lerp_speed * dt)
				current_absorb = math_step(current_absorb, self[var_name], step_speed * dt)
				update_absorb = true
			end

			if update_absorb and current_absorb > 0 then
				local shield_ratio = current_shield == 0 and 0 or math_min(current_absorb / current_shield, 1)
				local health_ratio = current_health == 0 and 0 or math_min((current_absorb - shield_ratio * current_shield) / current_health, 1)
				local shield = math_clamp(shield_ratio, 0, 1)
                local health = math_clamp(health_ratio, 0, 1)

                armor:set_w(bar_w * shield)
                hp:set_w(bar_w * health)
			end
		end
	end
end